@@ -69,6 +69,8 @@ struct _RBEncoderGstPrivate {
 	gint64 total_length;
 	guint progress_id;
 	char *dest_uri;
+
+	GOutputStream *outstream;
 };
 
 G_DEFINE_TYPE_WITH_CODE(RBEncoderGst, rb_encoder_gst, G_TYPE_OBJECT,
@@ -157,6 +159,12 @@ rb_encoder_gst_finalize (GObject *object)
 		encoder->priv->pipeline = NULL;
 	}
 
+	if (encoder->priv->outstream) {
+		g_output_stream_close (encoder->priv->outstream, NULL, NULL);
+		g_object_unref (encoder->priv->outstream);
+		encoder->priv->outstream = NULL;
+	}
+
 	g_free (encoder->priv->dest_uri);
 
         G_OBJECT_CLASS (rb_encoder_gst_parent_class)->finalize (object);
@@ -255,6 +263,16 @@ bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 		rb_debug ("received EOS");
 
 		gst_element_set_state (encoder->priv->pipeline, GST_STATE_NULL);
+		if (encoder->priv->outstream != NULL) {
+			GError *error = NULL;
+			g_output_stream_close (encoder->priv->outstream, NULL, &error);
+			if (error != NULL) {
+				rb_debug ("error closing output stream: %s", error->message);
+				g_error_free (error);
+			}
+			g_object_unref (encoder->priv->outstream);
+			encoder->priv->outstream = NULL;
+		}
 
 		rb_encoder_gst_emit_completed (encoder);
 
@@ -351,18 +369,18 @@ add_encoding_pipeline (RBEncoderGst *encoder,
 	GstPad *pad;
 	char *tmp;
 
-	queue = gst_element_factory_make ("queue", NULL);
+	queue = gst_element_factory_make ("queue2", NULL);
 	if (queue == NULL)
 		return NULL;
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), queue);
 
-	queue2 = gst_element_factory_make ("queue", NULL);
+	queue2 = gst_element_factory_make ("queue2", NULL);
 	if (queue2 == NULL)
 		return NULL;
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), queue2);
 
 	/* Nice big buffers... */
-	g_object_set (queue, "max-size-time", 120 * GST_SECOND, NULL);
+	g_object_set (queue, "max-size-time", 30 * GST_SECOND, "max-size-buffers", 0, "max-size-bytes", 0, NULL);
 
 	tmp = g_strdup_printf (GST_ENCODING_PROFILE, gm_audio_profile_get_pipeline (profile));
 	rb_debug ("constructing encoding bin from pipeline string %s", tmp);
@@ -394,6 +412,16 @@ add_encoding_pipeline (RBEncoderGst *encoder,
 	return queue2;
 }
 
+static void
+add_string_tag (GstTagList *tags, GstTagMergeMode mode, const char *tag, RhythmDBEntry *entry, RhythmDBPropType property)
+{
+	const char *v;
+	v = rhythmdb_entry_get_string (entry, property);
+	if (v != NULL && v[0] != '\0') {
+		gst_tag_list_add (tags, mode, tag, v, NULL);
+	}
+}
+
 static gboolean
 add_tags_from_entry (RBEncoderGst *encoder,
 		     RhythmDBEntry *entry,
@@ -406,17 +434,17 @@ add_tags_from_entry (RBEncoderGst *encoder,
 	tags = gst_tag_list_new ();
 
 	gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE_ALL,
-			  /* TODO: compute replay-gain */
-			  GST_TAG_TITLE, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE),
-			  GST_TAG_ARTIST, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST),
 			  GST_TAG_TRACK_NUMBER, rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER),
 			  GST_TAG_ALBUM_VOLUME_NUMBER, rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER),
-			  GST_TAG_ALBUM, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM),
-			  GST_TAG_GENRE, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE),
 			  GST_TAG_ENCODER, "Rhythmbox",
 			  GST_TAG_ENCODER_VERSION, VERSION,
 			  NULL);
 
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_TITLE, entry, RHYTHMDB_PROP_TITLE);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_ARTIST, entry, RHYTHMDB_PROP_ARTIST);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_ALBUM, entry, RHYTHMDB_PROP_ALBUM);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_GENRE, entry, RHYTHMDB_PROP_GENRE);
+
 	day = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DATE);
 
 	if (day > 0) {
@@ -428,33 +456,12 @@ add_tags_from_entry (RBEncoderGst *encoder,
 				  NULL);
 		g_date_free (date);
 	}
-#ifdef GST_TAG_MUSICBRAINZ_TRACKID
-	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID)) {
-		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
-				  GST_TAG_MUSICBRAINZ_TRACKID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID),
-				  NULL);
-	}
-	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID)) {
-		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
-				  GST_TAG_MUSICBRAINZ_ARTISTID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID),
-				  NULL);
-	}
-	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID)) {
-		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
-				  GST_TAG_MUSICBRAINZ_ALBUMID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID),
-				  NULL);
-	}
-	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID)) {
-		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
-				  GST_TAG_MUSICBRAINZ_ALBUMARTISTID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID),
-				  NULL);
-	}
-	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME)) {
-		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
-				  GST_TAG_MUSICBRAINZ_SORTNAME, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME),
-				  NULL);
-	}
-#endif
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_MUSICBRAINZ_TRACKID, entry, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_MUSICBRAINZ_ARTISTID, entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_MUSICBRAINZ_ALBUMID, entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_MUSICBRAINZ_ALBUMARTISTID, entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_ARTIST_SORTNAME, entry, RHYTHMDB_PROP_ARTIST_SORTNAME);
+	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_ALBUM_SORTNAME, entry, RHYTHMDB_PROP_ALBUM_SORTNAME);
 
 	{
 		GstIterator *iter;
@@ -516,7 +523,7 @@ new_decoded_pad_cb (GstElement *decodebin, GstPad *new_pad, gboolean arg1, RBEnc
 	/* only process audio data */
 	if (strncmp (caps_string, "audio/", 6) == 0) {
 		encoder->priv->decoded_pads++;
-		enc_sinkpad = gst_element_get_pad (encoder->priv->enc,
+		enc_sinkpad = gst_element_get_static_pad (encoder->priv->enc,
 				"sink");
 		if (gst_pad_link (new_pad, enc_sinkpad) != GST_PAD_LINK_OK)
 			rb_debug ("error linking pads");
@@ -534,7 +541,7 @@ add_decoding_pipeline (RBEncoderGst *encoder,
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	encoder->priv->transcoding = TRUE;
-	decodebin = gst_element_factory_make ("decodebin", NULL);
+	decodebin = gst_element_factory_make ("decodebin2", NULL);
 	if (decodebin == NULL) {
 		g_set_error (error,
 				RB_ENCODER_ERROR,
@@ -643,6 +650,7 @@ attach_output_pipeline (RBEncoderGst *encoder,
 
 		if (stream != NULL) {
 			g_object_set (sink, "stream", stream, NULL);
+			encoder->priv->outstream = G_OUTPUT_STREAM (stream);
 		}
 	} else {
 		rb_debug ("unable to create giostreamsink, falling back to default sink for %s", dest);
@@ -674,7 +682,7 @@ encoder_match_mime (RBEncoderGst *rbencoder, GstElement *encoder, const gchar *m
 	gboolean match = FALSE;
 	char *tmp;
 
-	srcpad = gst_element_get_pad (encoder, "src");
+	srcpad = gst_element_get_static_pad (encoder, "src");
 	element_caps = gst_pad_get_caps (srcpad);
 
 	if (element_caps == NULL) {
@@ -818,17 +826,23 @@ get_profile_from_mime_type (RBEncoderGst *rbencoder, const char *mime_type)
 static GMAudioProfile*
 get_profile_from_mime_types (RBEncoderGst *rbencoder, GList *mime_types)
 {
+	GMAudioProfile *profile = NULL;
 	GList *l;
 
-	for (l = mime_types; l != NULL; l = g_list_next (l)) {
-		GMAudioProfile *profile;
+	if (mime_types == NULL) {
+		const char *profile_name;
 
-		profile = get_profile_from_mime_type (rbencoder, (const char *)l->data);
-		if (profile)
-			return profile;
+		profile_name = eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT);
+		profile = gm_audio_profile_lookup (profile_name);
+	} else {
+		for (l = mime_types; l != NULL; l = g_list_next (l)) {
+			profile = get_profile_from_mime_type (rbencoder, (const char *)l->data);
+			if (profile != NULL)
+				break;
+		}
 	}
 
-	return NULL;
+	return profile;
 }
 
 static GstElement *
@@ -858,10 +872,10 @@ create_pipeline_and_source (RBEncoderGst *encoder,
 	encoder->priv->pipeline = gst_pipeline_new ("pipeline");
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), src);
 
-	/* TODO: add progress reporting */
+	/* provide a hook for setting source properties */
+	_rb_encoder_emit_prepare_source (RB_ENCODER (encoder), uri, G_OBJECT (src));
 
 	g_free (uri);
-
 	return src;
 }
 
@@ -889,73 +903,6 @@ copy_track (RBEncoderGst *encoder,
 	return TRUE;
 }
 
-static gboolean
-extract_track (RBEncoderGst *encoder,
-	       RhythmDBEntry *entry,
-	       const char *dest,
-	       GError **error)
-{
-	/* cdsrc ! encoder ! sink */
-	char *uri;
-	const char *device;
-	const char *profile_name;
-	GMAudioProfile *profile;
-	GstElement *src, *end;
-
-	g_assert (encoder->priv->pipeline == NULL);
-
-	profile_name = eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT);
-	profile = gm_audio_profile_lookup (profile_name);
-	if (profile == NULL) {
-		g_set_error (error,
-			     RB_ENCODER_ERROR, RB_ENCODER_ERROR_FORMAT_UNSUPPORTED,
-			     "Could not find encoding profile '%s'", profile_name);
-		return FALSE;
-	}
-
-	src = create_pipeline_and_source (encoder, entry, error);
-	if (src == NULL)
-		return FALSE;
-
-	/* setup cd extraction properties */
-	uri = rhythmdb_entry_get_playback_uri (entry);
-	if (uri == NULL) {
-		rb_debug ("didn't get a playback URI for entry %s",
-			  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
-		return FALSE;
-	}
-
-	device = g_utf8_strrchr (uri, -1, '#');
-	g_object_set (G_OBJECT (src),
-		      "device", device + 1, /* skip the '#' */
-		      "track", rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER),
-		      NULL);
-	if (g_object_class_find_property (G_OBJECT_GET_CLASS (src), "paranoia-mode")) {
-		int paranoia_mode;
-
-		paranoia_mode = 255; /* TODO: make configurable */
-		g_object_set (G_OBJECT (src), "paranoia-mode", paranoia_mode, NULL);
-	}
-	g_free (uri);
-
-	end = add_encoding_pipeline (encoder, profile, error);
-	if (end == NULL)
-		return FALSE;
-	if (gst_element_link (src, encoder->priv->enc) == FALSE) {
-		rb_debug ("unable to link source to encoding pipeline");
-		return FALSE;
-	}
-
-	if (!attach_output_pipeline (encoder, end, dest, error))
-		return FALSE;
-	if (!add_tags_from_entry (encoder, entry, error))
-		return FALSE;
-	if (!start_pipeline (encoder, error))
-		return FALSE;
-
-	return TRUE;
-}
-
 static gboolean
 transcode_track (RBEncoderGst *encoder,
 	 	 RhythmDBEntry *entry,
@@ -978,8 +925,7 @@ transcode_track (RBEncoderGst *encoder,
 			     /*"'%s'", mime_type*/);
 		goto error;
 	} else {
-		rb_debug ("selected profile %s",
-				gm_audio_profile_get_name (profile));
+		rb_debug ("selected profile %s", gm_audio_profile_get_name (profile));
 	}
 
 	src = create_pipeline_and_source (encoder, entry, error);
@@ -1026,6 +972,17 @@ rb_encoder_gst_cancel (RBEncoder *encoder)
 	g_object_unref (priv->pipeline);
 	priv->pipeline = NULL;
 
+	if (priv->outstream != NULL) {
+		GError *error = NULL;
+		g_output_stream_close (priv->outstream, NULL, &error);
+		if (error != NULL) {
+			rb_debug ("error closing output stream: %s", error->message);
+			g_error_free (error);
+		}
+		g_object_unref (priv->outstream);
+		priv->outstream = NULL;
+	}
+
 	rb_encoder_gst_emit_completed (RB_ENCODER_GST (encoder));
 }
 
@@ -1100,11 +1057,7 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 		priv->total_length = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
 		priv->position_format = GST_FORMAT_TIME;
 
-		if (mime_types == NULL) {
-			result = extract_track (RB_ENCODER_GST (encoder), entry, dest, &error);
-		} else {
-			result = transcode_track (RB_ENCODER_GST (encoder), entry, dest, mime_types, &error);
-		}
+		result = transcode_track (RB_ENCODER_GST (encoder), entry, dest, mime_types, &error);
 	}
 
 	if (error) {
@@ -1115,8 +1068,7 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 		if (enc->priv->pipeline == NULL) {
 			rb_encoder_gst_emit_completed (enc);
 		} else {
-			/* this will unref the pipeline and call emit_completed
-			 */
+			/* this will unref the pipeline and call emit_completed */
 			rb_encoder_gst_cancel (encoder);
 		}
 	}
