@@ -6,7 +6,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -83,6 +83,7 @@ static GPid metadata_child = 0;
 static int metadata_stdout = -1;
 static GMainContext *main_context = NULL;
 static GStaticMutex conn_mutex = G_STATIC_MUTEX_INIT;
+static char **saveable_types = NULL;
 
 struct RBMetaDataPrivate
 {
@@ -90,6 +91,9 @@ struct RBMetaDataPrivate
 	char       *mimetype;
 	char      **missing_plugins;
 	char      **plugin_descriptions;
+	gboolean    has_audio;
+	gboolean    has_video;
+	gboolean    has_other_data;
 	GHashTable *metadata;
 };
 
@@ -204,9 +208,13 @@ static gboolean
 start_metadata_service (GError **error)
 {
 	DBusError dbus_error = {0,};
+	DBusMessage *message;
+	DBusMessage *response;
+	DBusMessageIter iter;
 	GIOChannel *stdout_channel;
 	GIOStatus status;
 	gchar *dbus_address = NULL;
+	char *saveable_type_list;
 
 	if (dbus_connection) {
 		if (ping_metadata_service (error))
@@ -244,7 +252,7 @@ start_metadata_service (GError **error)
 		 * but when --enable-uninstalled-build is specified, we look
 		 * in the directory it's built in.
 		 */
-#ifdef METADATA_UNINSTALLED_DIR
+#ifdef USE_UNINSTALLED_DIRS
 		g_ptr_array_add (argv, METADATA_UNINSTALLED_DIR "/rhythmbox-metadata");
 #else
 		g_ptr_array_add (argv, LIBEXEC_DIR G_DIR_SEPARATOR_S INSTALLED_METADATA_HELPER);
@@ -304,6 +312,55 @@ start_metadata_service (GError **error)
 	dbus_connection_setup_with_g_main (dbus_connection, main_context);
 
 	rb_debug ("Metadata process %d started", metadata_child);
+
+	/* now ask it what types it can re-tag */
+	if (saveable_types != NULL) {
+		g_strfreev (saveable_types);
+	}
+
+	message = dbus_message_new_method_call (RB_METADATA_DBUS_NAME,
+						RB_METADATA_DBUS_OBJECT_PATH,
+						RB_METADATA_DBUS_INTERFACE,
+						"getSaveableTypes");
+	if (!message) {
+		/* what now? */
+		rb_debug ("unable to query metadata helper for saveable types");
+		return FALSE;
+	}
+
+	rb_debug ("sending metadata saveable types query");
+	response = dbus_connection_send_with_reply_and_block (dbus_connection,
+							      message,
+							      RB_METADATA_DBUS_TIMEOUT,
+							      &dbus_error);
+
+	if (!response) {
+		rb_debug ("saveable type query failed");
+		return FALSE;
+	}
+
+	if (!dbus_message_iter_init (response, &iter)) {
+		rb_debug ("couldn't read saveable type query response");
+		return FALSE;
+	}
+
+	if (!rb_metadata_dbus_get_strv (&iter, &saveable_types)) {
+		rb_debug ("couldn't get saveable type data from response message");
+		return FALSE;
+	}
+
+	if (saveable_types != NULL) {
+		saveable_type_list = g_strjoinv (", ", saveable_types);
+		rb_debug ("saveable types from metadata helper: %s", saveable_type_list);
+		g_free (saveable_type_list);
+	} else {
+		rb_debug ("unable to save metadata for any file types");
+	}
+
+	if (message)
+		dbus_message_unref (message);
+	if (response)
+		dbus_message_unref (response);
 	return TRUE;
 }
 
@@ -371,6 +428,11 @@ rb_metadata_load (RBMetaData *md,
 	DBusError dbus_error = {0,};
 	gboolean ok;
 	GError *fake_error = NULL;
+	GError *dbus_gerror;
+
+	dbus_gerror = g_error_new (RB_METADATA_ERROR,
+				   RB_METADATA_ERROR_INTERNAL,
+				   _("D-BUS communication error"));
 
 	if (error == NULL)
 		error = &fake_error;
@@ -397,15 +459,9 @@ rb_metadata_load (RBMetaData *md,
 							RB_METADATA_DBUS_INTERFACE,
 							"load");
 		if (!message) {
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_INTERNAL,
-				     _("D-BUS communication error"));
+			g_propagate_error (error, dbus_gerror);
 		} else if (!dbus_message_append_args (message, DBUS_TYPE_STRING, &uri, DBUS_TYPE_INVALID)) {
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_INTERNAL,
-				     _("D-BUS communication error"));
+			g_propagate_error (error, dbus_gerror);
 		}
 	}
 
@@ -422,30 +478,21 @@ rb_metadata_load (RBMetaData *md,
 
 	if (*error == NULL) {
 		if (!dbus_message_iter_init (response, &iter)) {
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_INTERNAL,
-				     _("D-BUS communication error"));
+			g_propagate_error (error, dbus_gerror);
 			rb_debug ("couldn't read response message");
 		}
 	}
 	
 	if (*error == NULL) {
 		if (!rb_metadata_dbus_get_strv (&iter, &md->priv->missing_plugins)) {
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_INTERNAL,
-				     _("D-BUS communication error"));
+			g_propagate_error (error, dbus_gerror);
 			rb_debug ("couldn't get missing plugin data from response message");
 		}
 	}
 
 	if (*error == NULL) {
 		if (!rb_metadata_dbus_get_strv (&iter, &md->priv->plugin_descriptions)) {
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_INTERNAL,
-				     _("D-BUS communication error"));
+			g_propagate_error (error, dbus_gerror);
 			rb_debug ("couldn't get missing plugin descriptions from response message");
 		}
 	}
@@ -459,30 +506,51 @@ rb_metadata_load (RBMetaData *md,
 		kill_metadata_service ();
 	}
 
+	if (*error == NULL) {
+		if (!rb_metadata_dbus_get_boolean (&iter, &md->priv->has_audio)) {
+			g_propagate_error (error, dbus_gerror);
+			rb_debug ("couldn't get has-audio flag from response message");
+		} else {
+			rb_debug ("has audio: %d", md->priv->has_audio);
+		}
+	}
+
+	if (*error == NULL) {
+		if (!rb_metadata_dbus_get_boolean (&iter, &md->priv->has_video)) {
+			g_propagate_error (error, dbus_gerror);
+			rb_debug ("couldn't get has-video flag from response message");
+		} else {
+			rb_debug ("has video: %d", md->priv->has_video);
+		}
+	}
+
+	if (*error == NULL) {
+		if (!rb_metadata_dbus_get_boolean (&iter, &md->priv->has_other_data)) {
+			g_propagate_error (error, dbus_gerror);
+			rb_debug ("couldn't get has-other-data flag from response message");
+		} else {
+			rb_debug ("has other data: %d", md->priv->has_other_data);
+		}
+	}
+
+	if (*error == NULL) {
+		if (!rb_metadata_dbus_get_string (&iter, &md->priv->mimetype)) {
+			g_propagate_error (error, dbus_gerror);
+		} else {
+			rb_debug ("got mimetype: %s", md->priv->mimetype);
+		}
+	}
+
 	if (*error == NULL) {
 		if (!rb_metadata_dbus_get_boolean (&iter, &ok)) {
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_INTERNAL,
-				     _("D-BUS communication error"));
+			g_propagate_error (error, dbus_gerror);
 			rb_debug ("couldn't get success flag from response message");
 		} else if (ok == FALSE) {
 			read_error_from_message (md, &iter, error);
 		}
 	}
 
-	if (ok && *error == NULL) {
-		if (!rb_metadata_dbus_get_string (&iter, &md->priv->mimetype)) {
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_INTERNAL,
-				     _("D-BUS communication error"));
-		} else {
-			rb_debug ("got mimetype: %s", md->priv->mimetype);
-		}
-	}
-
-	if (ok && *error == NULL) {
+	if (*error == NULL) {
 		rb_metadata_dbus_read_from_message (md, md->priv->metadata, &iter);
 	}
 
@@ -490,6 +558,8 @@ rb_metadata_load (RBMetaData *md,
 		dbus_message_unref (message);
 	if (response)
 		dbus_message_unref (response);
+	if (*error != dbus_gerror)
+		g_error_free (dbus_gerror);
 	if (fake_error)
 		g_error_free (fake_error);
 
@@ -626,52 +696,45 @@ gboolean
 rb_metadata_can_save (RBMetaData *md, const char *mimetype)
 {
 	GError *error = NULL;
-	DBusMessage *message = NULL;
-	DBusMessage *response = NULL;
-	gboolean can_save = FALSE;
-	DBusError dbus_error = {0,};
-	DBusMessageIter iter;
-	gboolean ok = TRUE;
+	gboolean result = FALSE;
+	int i = 0;
 
 	g_static_mutex_lock (&conn_mutex);
 
-	if (start_metadata_service (&error) == FALSE) {
-		g_error_free (error);
-		ok = FALSE;
-	}
-
-	if (ok) {
-		message = dbus_message_new_method_call (RB_METADATA_DBUS_NAME,
-							RB_METADATA_DBUS_OBJECT_PATH,
-							RB_METADATA_DBUS_INTERFACE,
-							"canSave");
-		if (!message) {
-			ok = FALSE;
-		} else if (!dbus_message_append_args (message, DBUS_TYPE_STRING, &mimetype, DBUS_TYPE_INVALID)) {
-			ok = FALSE;
+	if (saveable_types == NULL) {
+		if (start_metadata_service (&error) == FALSE) {
+			g_error_free (error);
+			return FALSE;
 		}
 	}
 
-	if (ok) {
-		response = dbus_connection_send_with_reply_and_block (dbus_connection,
-								      message,
-								      RB_METADATA_DBUS_TIMEOUT,
-								      &dbus_error);
-		if (!response) {
-			dbus_error_free (&dbus_error);
-			ok = FALSE;
-		} else if (dbus_message_iter_init (response, &iter)) {
-			rb_metadata_dbus_get_boolean (&iter, &can_save);
+	if (saveable_types != NULL) {
+		for (i = 0; saveable_types[i] != NULL; i++) {
+			if (g_str_equal (mimetype, saveable_types[i])) {
+				result = TRUE;
+				break;
+			}
 		}
 	}
 
-	if (message)
-		dbus_message_unref (message);
-	if (response)
-		dbus_message_unref (response);
 	g_static_mutex_unlock (&conn_mutex);
+	return result;
+}
 
-	return can_save;
+/**
+ * rb_metadata_get_saveable_types:
+ * @md: a #RBMetaData
+ *
+ * Constructs a list of the media types for which the metadata backend
+ * implements tag saving.
+ *
+ * Return value: a NULL-terminated array of media type strings.  Use g_strfreev
+ *  to free it.
+ */
+char **
+rb_metadata_get_saveable_types (RBMetaData *md)
+{
+	return g_strdupv (saveable_types);
 }
 
 /**
@@ -724,7 +787,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 	if (*error == NULL) {
 		response = dbus_connection_send_with_reply_and_block (dbus_connection,
 								      message,
-								      RB_METADATA_DBUS_TIMEOUT,
+								      RB_METADATA_SAVE_DBUS_TIMEOUT,
 								      &dbus_error);
 		if (!response) {
 			handle_dbus_error (md, &dbus_error, error);
@@ -743,3 +806,21 @@ rb_metadata_save (RBMetaData *md, GError **error)
 
 	g_static_mutex_unlock (&conn_mutex);
 }
+
+gboolean
+rb_metadata_has_audio (RBMetaData *md)
+{
+	return md->priv->has_audio;
+}
+
+gboolean
+rb_metadata_has_video (RBMetaData *md)
+{
+	return md->priv->has_video;
+}
+
+gboolean
+rb_metadata_has_other_data (RBMetaData *md)
+{
+	return md->priv->has_other_data;
+}
