@@ -7,7 +7,7 @@
 # the Free Software Foundation; either version 2, or (at your option)
 # any later version.
 #
-# The Rhythmbox authors hereby grants permission for non-GPL compatible
+# The Rhythmbox authors hereby grant permission for non-GPL compatible
 # GStreamer plugins to be used and distributed together with GStreamer
 # and Rhythmbox. This permission is above and beyond the permissions granted
 # by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -36,7 +36,7 @@ FADE_TOTAL_TIME = 1000
 ART_MISSING_ICON = 'rhythmbox-missing-artwork'
 WORKING_DELAY = 500
 THROBBER_RATE = 10
-THROBBER = 'gnome-spinner'
+THROBBER = 'process-working'
 ASPECT_RATIO_MIN = 0.9
 ASPECT_RATIO_MAX = 1.1
 
@@ -82,7 +82,10 @@ def merge_with_background (pixbuf, bgcolor, pad_if_not_near_square):
 	return ret
 
 class FadingImage (gtk.Misc):
-	__gsignals__ = { 'size-allocate': 'override' }
+	__gsignals__ = {
+		'size-allocate': 'override',
+		'get-max-size' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_INT, ())
+	}
 	def __init__ (self, missing_image):
 		gobject.GObject.__init__ (self)
 		self.sc_id = self.connect('screen-changed', self.screen_changed)
@@ -142,12 +145,17 @@ class FadingImage (gtk.Misc):
 		self.missing_pixbuf = merge_with_background (missing_pixbuf, self.style.bg[gtk.STATE_NORMAL], False)
 
 	def do_size_allocate (self, allocation):
+		old_width = self.allocation.width
 		self.allocation = allocation
+
 		if self.resize_id == 0:
 			self.resize_id = gobject.idle_add (self.after_resize)
-		if self.size != allocation.width:
-			self.size = allocation.width
+
+		if old_width != allocation.width:
+			max_size = self.emit ('get-max-size')
+			self.size = min (allocation.width, max_size)
 			self.queue_resize ()
+		
 		elif self.window is not None:
 			self.window.move_resize (allocation.x, allocation.y, allocation.width, allocation.height)
 			self.queue_draw ()
@@ -156,6 +164,7 @@ class FadingImage (gtk.Misc):
 	def after_resize (self):
 		self.reload_util_pixbufs ()
 		self.merged_pixbuf = None
+		self.resize_id = 0
 		self.queue_draw ()
 		return False
 
@@ -165,12 +174,23 @@ class FadingImage (gtk.Misc):
 	def expose (self, widget, event):
 		if not self.ensure_merged_pixbuf ():
 			return False
+
 		if self.merged_pixbuf.props.width != self.size:
 			draw_pb = self.merged_pixbuf.scale_simple (self.size, self.size, gtk.gdk.INTERP_NEAREST)
 		else:
 			draw_pb = self.merged_pixbuf
+
+		# center the image if we're wider than we are tall
 		x, y, w, h = event.area
-		event.window.draw_pixbuf (None, draw_pb, x, y, x, y, min (w, self.size - x), min (h, self.size - y))
+		pad = (self.allocation.width - self.size) / 2
+
+		left = max (x, pad)
+		right = min (x + w, pad + self.size)
+		top = y
+		bottom = min (y + h, self.size)
+		if right > left and bottom > top:
+			event.window.draw_pixbuf (None, draw_pb, left-pad, top, left, top, right - left, bottom - top)
+
 		if self.anim:
 			x, y, w, h = self.anim_rect ()
 			event.window.draw_pixbuf (None, self.anim, max (0, -x), max (0, -y), max (0, x), max (0, y), w, h)
@@ -255,9 +275,6 @@ class ArtDisplayWidget (FadingImage):
 		self.current_entry, self.working = None, False
 		self.current_pixbuf, self.current_uri = None, None
 
-		self.tooltips = gtk.Tooltips ()
-		self.tooltips.enable ()
-	
 	def disconnect_handlers (self):
  		super (ArtDisplayWidget, self).disconnect_handlers ()
 		self.disconnect (self.ddg_id)
@@ -286,11 +303,11 @@ class ArtDisplayWidget (FadingImage):
 
 	def update_tooltips (self, working):
 		if not self.current_entry:
-			self.tooltips.set_tip (self, None)
+			self.set_tooltip_text (None)
 		elif working:
-			self.tooltips.set_tip (self, _("Searching... drop artwork here"))
+			self.set_tooltip_text (_("Searching... drop artwork here"))
 		else:
-			self.tooltips.set_tip (self, _("Drop artwork here"))
+			self.set_tooltip_text (_("Drop artwork here"))
 
 	def set (self, entry, pixbuf, uri, working):
 		self.current_entry = entry
@@ -342,7 +359,10 @@ class ArtDisplayPlugin (rb.Plugin):
 		self.art_widget = ArtDisplayWidget (self.find_file (ART_MISSING_ICON + ".svg"))
 		self.art_widget.connect ('pixbuf-dropped', self.on_set_pixbuf)
 		self.art_widget.connect ('uri-dropped', self.on_set_uri)
-		shell.add_widget (self.art_widget, rb.SHELL_UI_LOCATION_SIDEBAR)
+		self.art_widget.connect ('get-max-size', self.get_max_art_size)
+		self.art_container = gtk.VBox ()
+		self.art_container.pack_start (self.art_widget, padding=6)
+		shell.add_widget (self.art_container, rb.SHELL_UI_LOCATION_SIDEBAR)
 		self.art_db = CoverArtDatabase ()
 		self.current_entry, self.current_pixbuf = None, None
 		self.playing_entry_changed (sp, sp.get_playing_entry ())
@@ -360,7 +380,7 @@ class ArtDisplayPlugin (rb.Plugin):
 			db.disconnect (id)
 		self.db_cb_ids = ()
 
-		shell.remove_widget (self.art_widget, rb.SHELL_UI_LOCATION_SIDEBAR)
+		shell.remove_widget (self.art_container, rb.SHELL_UI_LOCATION_SIDEBAR)
 		self.art_widget.disconnect_handlers ()
 		self.art_widget = None
 		self.art_db = None
@@ -377,18 +397,17 @@ class ArtDisplayPlugin (rb.Plugin):
 		db = self.shell.get_property ("db")
 
 		self.art_widget.set (entry, None, None, True)
-		self.art_widget.show ()
+		self.art_container.show_all ()
 		# Intitates search in the database (which checks art cache, internet etc.)
 		self.current_entry = entry
 		self.current_pixbuf = None
-		self.art_db.get_pixbuf(db, entry, self.on_get_pixbuf_completed)
+		self.art_db.get_pixbuf(db, entry, True, self.on_get_pixbuf_completed)
 
 	def on_get_pixbuf_completed(self, entry, pixbuf, uri):
 		# Set the pixbuf for the entry returned from the art db
-		if entry != self.current_entry:
-			return
-		self.current_pixbuf = pixbuf
-		self.art_widget.set (entry, pixbuf, uri, False)
+		if entry == self.current_entry:
+			self.current_pixbuf = pixbuf
+			self.art_widget.set (entry, pixbuf, uri, False)
 		if pixbuf:
 			db = self.shell.get_property ("db")
 			# This might be from a playing-changed signal,
@@ -399,8 +418,16 @@ class ArtDisplayPlugin (rb.Plugin):
 			gobject.idle_add(idle_emit_art)
 
 	def cover_art_request (self, db, entry):
-		if entry == self.current_entry:
-			return self.current_pixbuf
+		a = [None]
+		def callback(entry, pixbuf, uri):
+			a[0] = pixbuf
+			self.on_get_pixbuf_completed(entry, pixbuf, uri)
+
+		playing = (entry == self.current_entry)
+		self.art_db.get_pixbuf(db, entry, playing, callback)
+
+		# If callback was called synchronously we can return a pixmap
+		return a[0]
 
 	def cover_art_notify (self, db, entry, field, metadata):
 		if entry != self.current_entry:
@@ -436,7 +463,8 @@ class ArtDisplayPlugin (rb.Plugin):
 					pass
 
 		print "got cover art URI notification: %s" % (uri)
-		rb.Loader().get_url (uri, loader_cb)
+		l = rb.Loader()
+		l.get_url (uri, loader_cb)
 
 	def cover_art_uri_request (self, db, entry):
 		if entry == self.current_entry:
@@ -453,3 +481,10 @@ class ArtDisplayPlugin (rb.Plugin):
 	def on_set_uri (self, widget, entry, uri):
 		db = self.shell.get_property ("db")
 		self.art_db.set_pixbuf_from_uri (db, entry, uri, self.on_get_pixbuf_completed)
+
+	def get_max_art_size (self, widget):
+		# limit the art image to a third of the window height to prevent it from
+		# forcing the window to resize, obscuring everything else, and so on.
+		(width, height) = self.shell.props.window.get_size()
+		return height / 3
+
