@@ -78,6 +78,11 @@ static GList* impl_get_ui_actions (RBSource *source);
 
 static void impl_pack_paned (RBBrowserSource *source, GtkWidget *paned);
 
+#ifdef HAVE_SJ_METADATA_GETTER
+static void rb_audiocd_source_cmd_reload_metadata (GtkAction *action,
+						   RBAudioCdSource *source);
+#endif
+
 static gpointer rb_audiocd_load_songs (RBAudioCdSource *source);
 static void rb_audiocd_load_metadata (RBAudioCdSource *source, RhythmDB *db);
 static void rb_audiocd_load_metadata_cancel (RBAudioCdSource *source);
@@ -90,6 +95,9 @@ static gboolean update_album_cb (GtkWidget *widget, GdkEventFocus *event, RBAudi
 static gboolean update_genre_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
 static gboolean update_year_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
 static gboolean update_disc_number_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+static void info_bar_response_cb (GtkInfoBar *info_bar, gint response_id, RBAudioCdSource *source);
+#endif
 
 typedef struct
 {
@@ -109,7 +117,13 @@ typedef struct
 	GtkWidget *disc_number_entry;
 
 #ifdef HAVE_SJ_METADATA_GETTER
+	GtkActionGroup *action_group;
 	SjMetadataGetter *metadata;
+
+	GtkWidget *info_bar;
+	GtkWidget *info_bar_label;
+
+	char *submit_url;
 #endif
 } RBAudioCdSourcePrivate;
 
@@ -118,6 +132,13 @@ RB_PLUGIN_DEFINE_TYPE (RBAudioCdSource, rb_audiocd_source, RB_TYPE_REMOVABLE_MED
 
 #ifdef HAVE_SJ_METADATA_GETTER
 static AlbumDetails* multiple_album_dialog (GList *albums, RBAudioCdSource *source);
+
+static GtkActionEntry rb_audiocd_source_actions [] =
+{
+	{ "AudioCdSourceReloadMetadata", GTK_STOCK_REFRESH, N_("Reload"), NULL,
+	N_("Reload Album Information"),
+	G_CALLBACK (rb_audiocd_source_cmd_reload_metadata) },
+};
 #endif
 
 static RhythmDB *
@@ -182,6 +203,15 @@ rb_audiocd_source_finalize (GObject *object)
 
 	g_free (priv->device_path);
 
+#ifdef HAVE_SJ_METADATA_GETTER
+	if (priv->action_group != NULL) {
+		g_object_unref (priv->action_group);
+		priv->action_group = NULL;
+	}
+	g_free (priv->submit_url);
+	priv->submit_url = NULL;
+#endif
+
 	if (priv->tracks) {
 		g_list_free (priv->tracks);
 		priv->tracks = NULL;
@@ -208,6 +238,7 @@ rb_audiocd_source_constructor (GType type,
 			       guint n_construct_properties,
 			       GObjectConstructParam *construct_properties)
 {
+	RBAudioCdSourcePrivate *priv;
 	RBAudioCdSource *source;
 	RBEntryView *entry_view;
 	RhythmDB *db;
@@ -216,9 +247,17 @@ rb_audiocd_source_constructor (GType type,
 
 	source = RB_AUDIOCD_SOURCE (G_OBJECT_CLASS (rb_audiocd_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
+	priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
 	g_object_set (G_OBJECT (source), "name", "Unknown Audio", NULL);
 
+#ifdef HAVE_SJ_METADATA_GETTER
+	priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
+							       "AudiocdActions",
+							       rb_audiocd_source_actions,
+							       G_N_ELEMENTS (rb_audiocd_source_actions),
+							       source);
+#endif
 	/* we want audio cds to sort by track# by default */
 	entry_view = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_set_sorting_order (entry_view, "Track", GTK_SORT_ASCENDING);
@@ -252,6 +291,10 @@ rb_audiocd_source_constructor (GType type,
 		RBAudioCdSourcePrivate *priv;
 		GtkWidget *table;
 		GtkBuilder *builder;
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+		GtkWidget *box;
+		char *message;
+#endif
 
 		priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
@@ -261,6 +304,26 @@ rb_audiocd_source_constructor (GType type,
 		table = GTK_WIDGET (gtk_builder_get_object (builder, "album_info"));
 		g_assert (table != NULL);
 
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+		/* Info bar for non-Musicbrainz data */
+		priv->info_bar = gtk_info_bar_new_with_buttons (_("S_ubmit Album"), GTK_RESPONSE_OK,
+								_("Hide"), GTK_RESPONSE_CANCEL,
+								NULL);
+		message = g_strdup_printf ("<b>%s</b>\n%s", _("Could not find this album on MusicBrainz."),
+					   _("You can improve the MusicBrainz database by adding this album."));
+		priv->info_bar_label = gtk_label_new (NULL);
+		gtk_label_set_markup (GTK_LABEL (priv->info_bar_label), message);
+		gtk_label_set_justify (GTK_LABEL (priv->info_bar_label), GTK_JUSTIFY_LEFT);
+		g_free (message);
+		box = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->info_bar));
+		gtk_container_add (GTK_CONTAINER (box), priv->info_bar_label);
+		gtk_widget_show_all (box);
+		gtk_widget_set_no_show_all (priv->info_bar, TRUE);
+		g_signal_connect (G_OBJECT (priv->info_bar), "response",
+				  G_CALLBACK (info_bar_response_cb), source);
+		gtk_table_attach_defaults (GTK_TABLE (table), priv->info_bar, 0, 2, 0, 1);
+#endif
+
 		priv->artist_entry = GTK_WIDGET (gtk_builder_get_object (builder, "artist_entry"));
 		priv->artist_sort_entry = GTK_WIDGET (gtk_builder_get_object (builder, "artist_sort_entry"));
 		priv->album_entry = GTK_WIDGET (gtk_builder_get_object (builder, "album_entry"));
@@ -515,6 +578,15 @@ rb_audiocd_scan_songs (RBAudioCdSource *source,
 }
 
 #ifdef HAVE_SJ_METADATA_GETTER
+static void
+rb_audiocd_source_cmd_reload_metadata (GtkAction *action, RBAudioCdSource *source)
+{
+	RhythmDB *db;
+
+	db = get_db_for_source (source);
+	rb_audiocd_load_metadata (source, db);
+	g_object_unref (db);
+}
 
 /*
  * Called by the Multiple Album dialog when the user hits return in
@@ -672,6 +744,9 @@ metadata_cb (SjMetadataGetter *metadata,
 	g_value_init (&true_value, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&true_value, TRUE);
 
+	g_free (priv->submit_url);
+	priv->submit_url = NULL;
+
 	/* if we have multiple results, ask the user to pick one */
 	if (g_list_length (albums) > 1) {
 		album = multiple_album_dialog (albums, source);
@@ -680,6 +755,14 @@ metadata_cb (SjMetadataGetter *metadata,
 	} else
 		album = (AlbumDetails *)albums->data;
 
+#if GTK_CHECK_VERSION(2,17,6)
+	if (album->metadata_source != SOURCE_MUSICBRAINZ) {
+		priv->submit_url = sj_metadata_getter_get_submit_url (metadata);
+		if (priv->submit_url != NULL)
+			gtk_widget_show (priv->info_bar);
+	}
+#endif
+
 	if (album->metadata_source == SOURCE_FALLBACK) {
 		rb_debug ("ignoring CD metadata from fallback source");
 		g_object_unref (metadata);
@@ -953,6 +1036,10 @@ impl_get_ui_actions (RBSource *source)
 	actions = g_list_prepend (actions, g_strdup ("RemovableSourceCopyAllTracks"));
 	actions = g_list_prepend (actions, g_strdup ("RemovableSourceEject"));
 
+#ifdef HAVE_SJ_METADATA_GETTER
+	actions = g_list_prepend (actions, g_strdup ("AudioCdSourceReloadMetadata"));
+#endif
+
 	return actions;
 }
 
@@ -1137,3 +1224,24 @@ update_disc_number_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource
 	return FALSE;
 }
 
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+static void
+info_bar_response_cb (GtkInfoBar *info_bar, gint response_id, RBAudioCdSource *source)
+{
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+	GError *error = NULL;
+
+	g_return_if_fail (priv->submit_url != NULL);
+
+	if (response_id == GTK_RESPONSE_OK) {
+		if (!gtk_show_uri (NULL, priv->submit_url, GDK_CURRENT_TIME, &error)) {
+			rb_debug ("Could not launch submit URL %s: %s", priv->submit_url, error->message);
+			g_error_free (error);
+			return;
+		}
+	}
+
+	gtk_widget_hide (priv->info_bar);
+}
+#endif
+
