@@ -1321,7 +1321,6 @@ rb_daap_connection_connect (RBDAAPConnection        *connection,
 			    gpointer                 user_data)
 {
 	ConnectionResponseData *rdata;
-	char                   *path;
 
 	g_return_if_fail (RB_IS_DAAP_CONNECTION (connection));
 	g_return_if_fail (connection->priv->state == DAAP_GET_INFO);
@@ -1330,9 +1329,10 @@ rb_daap_connection_connect (RBDAAPConnection        *connection,
 
 	connection->priv->session = soup_session_async_new ();
 
-	path = g_strdup_printf ("http://%s:%d", connection->priv->host, connection->priv->port);
-	connection->priv->base_uri = soup_uri_new (path);
-	g_free (path);
+	connection->priv->base_uri = soup_uri_new (NULL);
+	soup_uri_set_scheme (connection->priv->base_uri, SOUP_URI_SCHEME_HTTP);
+	soup_uri_set_host (connection->priv->base_uri, connection->priv->host);
+	soup_uri_set_port (connection->priv->base_uri, connection->priv->port);
 
 	if (connection->priv->base_uri == NULL) {
 		rb_debug ("Error parsing http://%s:%d", connection->priv->host, connection->priv->port);
