@@ -2,13 +2,14 @@
  * rb-daap-plugin.c
  *
  * Copyright (C) 2006 James Livingston <doclivingston@gmail.com>
+ * Copyright (C) 2008 Alban Crequy <alban.crequy@collabora.co.uk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  *
- * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
  * GStreamer plugins to be used and distributed together with GStreamer
  * and Rhythmbox. This permission is above and beyond the permissions granted
  * by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -36,8 +37,8 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 #include <glib-object.h>
+#include <dbus/dbus-glib.h>
 
-#include "rb-soup-compat.h"
 #include <libsoup/soup.h>
 
 #include "rb-daap-plugin.h"
@@ -45,7 +46,7 @@
 #include "rb-shell.h"
 #include "rb-dialog.h"
 #include "rb-file-helpers.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "eel-gconf-extensions.h"
 #include "rb-daap-source.h"
 #include "rb-daap-sharing.h"
@@ -58,14 +59,17 @@
 #define CONF_DAAP_PREFIX  	CONF_PREFIX "/plugins/daap"
 #define CONF_ENABLE_BROWSING 	CONF_DAAP_PREFIX "/enable_browsing"
 
+#define DAAP_DBUS_PATH	"/org/gnome/Rhythmbox/DAAP"
+
 struct RBDaapPluginPrivate
 {
 	RBShell *shell;
 
-	GladeXML *config_xml;
+	GtkBuilder *builder;
 	GtkWidget *preferences;
 	gboolean sharing;
 	gboolean shutdown;
+	gboolean dbus_intf_added;
 
 	GtkActionGroup *daap_action_group;
 	guint daap_ui_merge_id;
@@ -94,7 +98,7 @@ static void rb_daap_plugin_get_property (GObject *object,
 					 GParamSpec *pspec);
 
 static void rb_daap_plugin_init (RBDaapPlugin *plugin);
-static void rb_daap_plugin_finalize (GObject *object);
+static void rb_daap_plugin_dispose (GObject *object);
 static void impl_activate (RBPlugin *plugin, RBShell *shell);
 static void impl_deactivate (RBPlugin *plugin, RBShell *shell);
 
@@ -110,6 +114,10 @@ static void enable_browsing_changed_cb (GConfClient *client,
 					GConfEntry *entry,
 					RBDaapPlugin *plugin);
 
+gboolean rb_daap_add_source (RBDaapPlugin *plugin, gchar *service_name, gchar *host, unsigned int port, GError **error);
+gboolean rb_daap_remove_source (RBDaapPlugin *plugin, gchar *service_name, GError **error);
+#include "rb-daap-glue.h"
+
 RB_PLUGIN_REGISTER(RBDaapPlugin, rb_daap_plugin)
 
 static GtkActionEntry rb_daap_source_actions [] =
@@ -117,7 +125,7 @@ static GtkActionEntry rb_daap_source_actions [] =
 	{ "DaapSourceDisconnect", GTK_STOCK_DISCONNECT, N_("_Disconnect"), NULL,
 	  N_("Disconnect from DAAP share"),
 	  G_CALLBACK (rb_daap_plugin_cmd_disconnect) },
-	{ "MusicNewDAAPShare", GTK_STOCK_CONNECT, N_("Connect to _DAAP share"), NULL,
+	{ "MusicNewDAAPShare", GTK_STOCK_CONNECT, N_("Connect to _DAAP share..."), NULL,
 	  N_("Connect to a new DAAP share"),
 	  G_CALLBACK (rb_daap_plugin_cmd_connect) },
 };
@@ -128,7 +136,7 @@ rb_daap_plugin_class_init (RBDaapPluginClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBPluginClass *plugin_class = RB_PLUGIN_CLASS (klass);
 
-	object_class->finalize = rb_daap_plugin_finalize;
+	object_class->dispose = rb_daap_plugin_dispose;
 	object_class->get_property = rb_daap_plugin_get_property;
 
 	plugin_class->activate = impl_activate;
@@ -156,16 +164,23 @@ rb_daap_plugin_init (RBDaapPlugin *plugin)
 }
 
 static void
-rb_daap_plugin_finalize (GObject *object)
+rb_daap_plugin_dispose (GObject *object)
 {
 	RBDaapPlugin *plugin = RB_DAAP_PLUGIN (object);
 
-	rb_debug ("RBDaapPlugin finalising");
+	rb_debug ("RBDaapPlugin dispose");
 
-	if (plugin->priv->preferences)
+	if (plugin->priv->preferences) {
 		gtk_widget_destroy (plugin->priv->preferences);
+		plugin->priv->preferences = NULL;
+	}
 
-	G_OBJECT_CLASS (rb_daap_plugin_parent_class)->finalize (object);
+	if (plugin->priv->builder) {
+		g_object_unref (plugin->priv->builder);
+		plugin->priv->builder = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_daap_plugin_parent_class)->dispose (object);
 }
 
 
@@ -198,7 +213,10 @@ impl_activate (RBPlugin *bplugin,
 	GConfClient *client = eel_gconf_client_get_global ();
 	GtkUIManager *uimanager = NULL;
 	char *uifile;
+	DBusGConnection *conn;
+	GError *error = NULL;
 
+	plugin->priv->shutdown = FALSE;
 	plugin->priv->shell = g_object_ref (shell);
 
 	value = gconf_client_get_without_default (client,
@@ -234,10 +252,6 @@ impl_activate (RBPlugin *bplugin,
 
 	/* add UI */
 	uifile = rb_plugin_find_file (bplugin, "daap-ui.xml");
-	if (uifile == NULL) {
-		uifile = g_strdup (rb_file ("daap-ui.xml"));
-	}
-
 	if (uifile != NULL) {
 		plugin->priv->daap_ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager, uifile, NULL);
 		g_free (uifile);
@@ -256,6 +270,22 @@ impl_activate (RBPlugin *bplugin,
 	plugin->priv->sharing = !no_registration;
 	if (plugin->priv->sharing)
 		rb_daap_sharing_init (shell);
+
+	/*
+	 * Add dbus interface
+	 */
+	if (plugin->priv->dbus_intf_added == FALSE) {
+		conn = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+		if (conn != NULL) {
+			dbus_g_object_type_install_info (RB_TYPE_DAAP_PLUGIN,
+							 &dbus_glib_rb_daap_object_info);
+			dbus_g_connection_register_g_object (conn, DAAP_DBUS_PATH,
+							     G_OBJECT (bplugin));
+			plugin->priv->dbus_intf_added = TRUE;
+		}
+		else
+			rb_debug ("No session D-Bus. DAAP interface on D-Bus disabled.");
+	}
 }
 
 static void
@@ -592,43 +622,6 @@ rb_daap_plugin_cmd_disconnect (GtkAction *action,
 	}
 }
 
-typedef struct {
-	RBDaapPlugin *plugin;
-	char *location;
-} RBDaapShareResolveData;
-
-static void
-new_daap_share_resolve_cb (SoupAddress *addr,
-			   guint status,
-			   RBDaapShareResolveData *data)
-{
-	GDK_THREADS_ENTER ();
-
-	if (status == SOUP_STATUS_OK) {
-		rb_debug ("adding manually specified DAAP share at %s", data->location);
-		mdns_service_added (NULL,
-				    data->location,
-				    data->location,
-				    soup_address_get_physical (addr),
-				    soup_address_get_port (addr),
-				    FALSE,
-				    data->plugin);
-	} else {
-		rb_debug ("unable to resolve DAAP share address %s",
-			  data->location);
-		rb_error_dialog (NULL, _("Could not connect to shared music"),
-				 _("Unable to resolve hostname %s"),
-				 data->location);
-	}
-
-	g_object_unref (data->plugin);
-	g_free (data->location);
-	g_free (data);
-	g_object_unref (addr);
-
-	GDK_THREADS_LEAVE ();
-}
-
 static void
 new_daap_share_location_added_cb (RBURIDialog *dialog,
 				  const char *location,
@@ -637,12 +630,6 @@ new_daap_share_location_added_cb (RBURIDialog *dialog,
 	char *host;
 	char *p;
 	int port = 3689;
-	SoupAddress *addr;
-	RBDaapShareResolveData *data;
-
-	data = g_new0 (RBDaapShareResolveData, 1);
-	data->plugin = g_object_ref (plugin);
-	data->location = g_strdup (location);
 
 	host = g_strdup (location);
 	p = strrchr (host, ':');
@@ -651,15 +638,17 @@ new_daap_share_location_added_cb (RBURIDialog *dialog,
 		*p = '\0';
 	}
 
-	addr = soup_address_new (host, port);
+	rb_debug ("adding manually specified DAAP share at %s", location);
+	mdns_service_added (NULL,
+			    g_strdup (location),
+			    g_strdup (location),
+			    g_strdup (host),
+			    port,
+			    FALSE,
+			    plugin);
+
 	g_free (host);
 
-	soup_address_resolve_async (addr,
-#if defined(HAVE_LIBSOUP_2_4)
-				    NULL, NULL,
-#endif
-				    (SoupAddressCallback) new_daap_share_resolve_cb,
-				    data);
 }
 
 static void
@@ -828,11 +817,11 @@ update_config_widget (RBDaapPlugin *plugin)
 	char *name;
 	char *password;
 
-	check = glade_xml_get_widget (plugin->priv->config_xml, "daap_enable_check");
-	password_check = glade_xml_get_widget (plugin->priv->config_xml, "daap_password_check");
-	name_entry = glade_xml_get_widget (plugin->priv->config_xml, "daap_name_entry");
-	password_entry = glade_xml_get_widget (plugin->priv->config_xml, "daap_password_entry");
-	box = glade_xml_get_widget (plugin->priv->config_xml, "daap_box");
+	check = GTK_WIDGET (gtk_builder_get_object (plugin->priv->builder, "daap_enable_check"));
+	password_check = GTK_WIDGET (gtk_builder_get_object (plugin->priv->builder, "daap_password_check"));
+	name_entry = GTK_WIDGET (gtk_builder_get_object (plugin->priv->builder, "daap_name_entry"));
+	password_entry = GTK_WIDGET (gtk_builder_get_object (plugin->priv->builder, "daap_password_entry"));
+	box = GTK_WIDGET (gtk_builder_get_object (plugin->priv->builder, "daap_box"));
 
 	sharing_enabled = eel_gconf_get_boolean (CONF_DAAP_ENABLE_SHARING);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), sharing_enabled);
@@ -865,21 +854,18 @@ update_config_widget (RBDaapPlugin *plugin)
 static GtkWidget *
 make_config_widget (RBDaapPlugin *plugin)
 {
-	char *gladefile;
+	char *builder_file;
 
-	gladefile = rb_plugin_find_file (RB_PLUGIN (plugin), "daap-prefs.glade");
-	if (gladefile == NULL) {
-		gladefile = g_strdup (rb_file ("daap-prefs.glade"));
-		if (gladefile == NULL) {
-			return NULL;
-		}
+	builder_file = rb_plugin_find_file (RB_PLUGIN (plugin), "daap-prefs.ui");
+	if (builder_file == NULL) {
+		return NULL;
 	}
 
-	plugin->priv->config_xml = glade_xml_new (gladefile, "daap_vbox", NULL);
-	g_free (gladefile);
+	plugin->priv->builder = rb_builder_load (builder_file, NULL);
+	g_free (builder_file);
 
 	update_config_widget (plugin);
-	return glade_xml_get_widget (plugin->priv->config_xml, "daap_vbox");
+	return GTK_WIDGET (gtk_builder_get_object (plugin->priv->builder, "daap_vbox"));
 }
 
 
@@ -913,3 +899,34 @@ impl_create_configure_dialog (RBPlugin *bplugin)
 	return plugin->priv->preferences;
 }
 
+gboolean
+rb_daap_add_source (RBDaapPlugin *plugin, gchar *service_name, gchar *host, unsigned int port, GError **error)
+{
+
+	if (plugin->priv->shutdown)
+		return FALSE;
+
+	rb_debug ("Add DAAP source %s (%s:%d)", service_name, host, port);
+
+	rb_debug ("adding manually specified DAAP share at %s", service_name);
+	mdns_service_added (NULL,
+			    g_strdup (service_name),
+			    g_strdup (service_name),
+			    g_strdup (host),
+			    port,
+			    FALSE,
+			    plugin);
+
+	return TRUE;
+}
+
+gboolean
+rb_daap_remove_source (RBDaapPlugin *plugin, gchar *service_name, GError **error)
+{
+	if (plugin->priv->shutdown)
+		return FALSE;
+	rb_debug ("Remove DAAP source %s", service_name);
+	mdns_service_removed (plugin->priv->mdns_browser, service_name, plugin);
+	return TRUE;
+}
+
