@@ -58,7 +58,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <glib-object.h>
-#include <glade/glade.h>
 
 #include <gst/gst.h>
 #include <gst/gstutils.h>
@@ -75,6 +74,7 @@
 #include "rb-file-helpers.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
+#include "rb-builder-helpers.h"
 
 #define VISUALIZATION_ICON_NAME	"visualization"
 
@@ -82,9 +82,6 @@
 
 #include "rb-vis-widget.h"
 
-/* not going to create a new header just for this */
-extern GType rb_fake_vis_get_type (void);
-
 /* preferences */
 #define CONF_VIS_PREFIX  CONF_PREFIX "/plugins/visualizer"
 #define CONF_VIS_ELEMENT CONF_VIS_PREFIX "/element"
@@ -108,6 +105,8 @@ extern GType rb_fake_vis_get_type (void);
 
 #define VISUALIZER_DBUS_PATH	"/org/gnome/Rhythmbox/Visualizer"
 
+/* playbin2 flag(s) */
+#define PLAYBIN2_FLAG_VIS	0x08
 
 typedef struct {
 	char *name;
@@ -240,13 +239,6 @@ static const VisualizerQuality vis_quality[] = {
 	{ N_("Extra Large"),	800,	600,	30,	1 },
 };
 
-/* "quality" to use for fake visualization; should be small and
- * low framerate, but not so slow it makes a noticeable difference
- * to playback start time.  the video sink needs to preroll too,
- * so if the framerate is 1/10, that will take 10 seconds.
- */
-static const VisualizerQuality fake_vis_quality = { "", 60, 60, 1, 1 };
-
 static const VisualizerModeName vis_mode_name[] = {
 	{ N_("Embedded"),	EMBEDDED },
 	{ N_("Fullscreen"),	FULLSCREEN },
@@ -256,6 +248,24 @@ static const VisualizerModeName vis_mode_name[] = {
 
 RB_PLUGIN_REGISTER(RBVisualizerPlugin, rb_visualizer_plugin)
 
+static GdkCursor *
+get_blank_cursor (GdkWindow *window)
+{
+#if !GTK_CHECK_VERSION(2,16,0)
+	GdkPixmap *pixmap;
+	GdkCursor *cursor;
+	GdkColor color = {0, 0, 0, 0};
+
+	pixmap = gdk_bitmap_create_from_data (window, "\0\0\0\0\0\0\0\0", 1, 1);
+	cursor = gdk_cursor_new_from_pixmap (pixmap, pixmap, &color, &color, 0, 0);
+	g_object_unref (pixmap);
+
+	return cursor;
+#else
+	return gdk_cursor_new (GDK_BLANK_CURSOR);
+#endif
+}
+
 static void
 rb_visualizer_plugin_init (RBVisualizerPlugin *plugin)
 {
@@ -425,12 +435,7 @@ bus_sync_message_cb (GstBus *bus, GstMessage *msg, RBVisualizerPlugin *plugin)
 	case EXTERNAL_WINDOW:
 		if (plugin->vis_widget != NULL) {
 			g_object_get (plugin->vis_widget, "window-xid", &window, NULL);
-			if (window == 0) {
-				window = GDK_WINDOW_XWINDOW (plugin->fake_window);
-				rb_debug ("setting fake window id %lu", window);
-			} else {
-				rb_debug ("setting window id %lu in prepare-xwindow-id handler", window);
-			}
+			rb_debug ("setting window id %lu in prepare-xwindow-id handler", window);
 		}
 		break;
 	case DESKTOP_WINDOW:
@@ -439,7 +444,7 @@ bus_sync_message_cb (GstBus *bus, GstMessage *msg, RBVisualizerPlugin *plugin)
 		break;
 	}
 
-	if (plugin->xoverlay != NULL)
+	if (plugin->xoverlay != NULL && window != 0)
 		gst_x_overlay_set_xwindow_id (plugin->xoverlay, window);
 	plugin->window_id_set = TRUE;
 }
@@ -459,12 +464,9 @@ fixate_vis_caps (RBVisualizerPlugin *pi, GstElement *vis_element, GstElement *ca
 	if (quality < 0 || quality > G_N_ELEMENTS (vis_quality))
 		quality = DEFAULT_VIS_QUALITY;
 
-	if (pi->active)
-		q = &vis_quality[quality];
-	else
-		q = &fake_vis_quality;
+	q = &vis_quality[quality];
 
-	pad = gst_element_get_pad (vis_element, "src");
+	pad = gst_element_get_static_pad (vis_element, "src");
 	template_caps = gst_pad_get_pad_template_caps (pad);
 	gst_object_unref (pad);
 
@@ -536,12 +538,24 @@ update_playbin_visualizer (RBVisualizerPlugin *plugin,
 {
 	GstPad *pad;
 	GstElement *vis_plugin;
+	int playbin_flags;
 
 	if (plugin->playbin == NULL)
 		return;
 
-	if (plugin->visualizer)
+	if (plugin->visualizer) {
 		g_object_unref (plugin->visualizer);
+		plugin->visualizer = NULL;
+	}
+
+	g_object_get (plugin->playbin, "flags", &playbin_flags, NULL);
+
+	if (plugin->active == FALSE) {
+		playbin_flags &= ~PLAYBIN2_FLAG_VIS;
+		rb_debug ("disabling vis; new playbin2 flags %d", playbin_flags);
+		g_object_set (plugin->playbin, "flags", playbin_flags, NULL);
+		return;
+	}
 
 	plugin->visualizer = gst_bin_new (NULL);
 
@@ -549,20 +563,15 @@ update_playbin_visualizer (RBVisualizerPlugin *plugin,
 	plugin->capsfilter = gst_element_factory_make ("capsfilter", NULL);
 	gst_bin_add (GST_BIN (plugin->visualizer), plugin->capsfilter);
 
-	pad = gst_element_get_pad (plugin->capsfilter, "src");
+	pad = gst_element_get_static_pad (plugin->capsfilter, "src");
 	gst_element_add_pad (plugin->visualizer, gst_ghost_pad_new ("src", pad));
 	gst_object_unref (pad);
 
 	/* set up visualizer */
-	if (plugin->active) {
-		vis_plugin = create_visualizer_element (vis_override);
-		gst_bin_add (GST_BIN (plugin->visualizer), vis_plugin);
-	} else {
-		vis_plugin = g_object_new (rb_fake_vis_get_type (), NULL);
-		gst_bin_add (GST_BIN (plugin->visualizer), vis_plugin);
-	}
+	vis_plugin = create_visualizer_element (vis_override);
+	gst_bin_add (GST_BIN (plugin->visualizer), vis_plugin);
 
-	pad = gst_element_get_pad (vis_plugin, "sink");
+	pad = gst_element_get_static_pad (vis_plugin, "sink");
 	gst_element_add_pad (plugin->visualizer, gst_ghost_pad_new ("sink", pad));
 	gst_object_unref (pad);
 
@@ -571,7 +580,12 @@ update_playbin_visualizer (RBVisualizerPlugin *plugin,
 
 	g_object_ref (plugin->visualizer);
 
-	g_object_set (plugin->playbin, "vis-plugin", plugin->visualizer, NULL);
+	playbin_flags |= PLAYBIN2_FLAG_VIS;
+	rb_debug ("enabling vis; new playbin2 flags %d", playbin_flags);
+	g_object_set (plugin->playbin,
+		      "vis-plugin", plugin->visualizer,
+		      "flags", playbin_flags,
+		      NULL);
 }
 
 static void
@@ -664,7 +678,7 @@ update_tee_visualizer (RBVisualizerPlugin *plugin,
 
 			/* probably should do this async.. */
 			rb_debug ("blocking visualizer bin sink pad");
-			pad = gst_element_get_pad (plugin->visualizer, "sink");
+			pad = gst_element_get_static_pad (plugin->visualizer, "sink");
 			blocked_pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
 			gst_pad_set_blocked (blocked_pad, TRUE);
 			gst_object_unref (pad);
@@ -809,6 +823,14 @@ actually_hide_controls (RBVisualizerPlugin *plugin)
 		 * to exit fullscreen mode.
 		 */
 		gtk_widget_grab_focus (plugin->vis_widget);
+
+		if (GTK_WIDGET_REALIZED (plugin->vis_widget)) {
+			GdkCursor *cursor;
+
+			cursor = get_blank_cursor (plugin->vis_widget->window);
+			gdk_window_set_cursor (plugin->vis_widget->window, cursor);
+			gdk_cursor_unref (cursor);
+		}
 		/* fall through */
 	case EMBEDDED:
 	case EXTERNAL_WINDOW:
@@ -861,6 +883,9 @@ show_controls (RBVisualizerPlugin *plugin, gboolean play_controls_only)
 		case FULLSCREEN:
 			gtk_widget_show (plugin->play_control_widget);
 			gtk_widget_show (plugin->disable_button);
+			if (GTK_WIDGET_REALIZED (plugin->vis_widget)) {
+				gdk_window_set_cursor (plugin->vis_widget->window, NULL);
+			}
 			break;
 		case EXTERNAL_WINDOW:
 			gtk_widget_hide (plugin->play_control_widget);
@@ -1081,6 +1106,11 @@ update_window (RBVisualizerPlugin *plugin, VisualizerMode mode, int screen, int
 		gtk_box_pack_start (GTK_BOX (plugin->vis_box), plugin->play_control_widget, FALSE, FALSE, 6);
 		gtk_box_pack_end (GTK_BOX (plugin->vis_box), plugin->control_widget, FALSE, FALSE, 6);
 	} else {
+		if (plugin->xoverlay != NULL) {
+			gst_x_overlay_set_xwindow_id (plugin->xoverlay,
+						      GDK_WINDOW_XWINDOW (plugin->fake_window));
+		}
+
 		switch (plugin->mode) {
 		case EMBEDDED:
 			gtk_container_remove (GTK_CONTAINER (plugin->vis_shell),
@@ -1210,7 +1240,9 @@ update_window (RBVisualizerPlugin *plugin, VisualizerMode mode, int screen, int
 
 		rb_shell_add_widget (plugin->shell,
 				     plugin->vis_box,
-				     RB_SHELL_UI_LOCATION_MAIN_BOTTOM);
+				     RB_SHELL_UI_LOCATION_MAIN_BOTTOM,
+				     FALSE,
+				     TRUE);
 
 		if (plugin->xoverlay != NULL) {
 			gst_x_overlay_set_xwindow_id (plugin->xoverlay, plugin->remote_window);
@@ -1584,8 +1616,6 @@ impl_activate (RBPlugin *plugin,
 	GtkAction *action;
 	char *ui_file;
 
-	rb_fake_vis_get_type ();
-
 	pi->shell = shell;
 
 	/* find the player backend and connect to its pipeline mutation signal */
@@ -1647,7 +1677,7 @@ impl_activate (RBPlugin *plugin,
 		gst_element_link_many (pi->capsfilter, colorspace, videoscale, pi->video_sink, NULL);
 		/* leave identity unlinked until we have a visualizer element */
 
-		pad = gst_element_get_pad (pi->identity, "sink");
+		pad = gst_element_get_static_pad (pi->identity, "sink");
 		gst_element_add_pad (pi->visualizer, gst_ghost_pad_new ("sink", pad));
 		gst_object_unref (pad);
 
@@ -1678,15 +1708,15 @@ impl_activate (RBPlugin *plugin,
 							NULL);
 	g_free (ui_file);
 
-	g_object_unref (uim);
 
 	if (pi->vis_shell == NULL) {
 		pi->vis_shell = gtk_vbox_new (FALSE, 0);
-		rb_shell_add_widget (pi->shell, pi->vis_shell, RB_SHELL_UI_LOCATION_MAIN_NOTEBOOK);
+		rb_shell_add_widget (pi->shell, pi->vis_shell, RB_SHELL_UI_LOCATION_MAIN_NOTEBOOK, FALSE, TRUE);
 	}
 
 	if (pi->vis_window == NULL) {
 		GtkWindow *parent;
+		GtkAccelGroup *accel_group;
 
 		pi->vis_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 		gtk_window_set_title (GTK_WINDOW (pi->vis_window), _("Music Player Visualization"));
@@ -1701,8 +1731,13 @@ impl_activate (RBPlugin *plugin,
 					 "delete-event",
 					 G_CALLBACK (window_delete_cb),
 					 pi, 0);
+
+		accel_group = gtk_ui_manager_get_accel_group (uim);
+		gtk_window_add_accel_group (GTK_WINDOW (pi->vis_window), accel_group);
 	}
 
+	g_object_unref (uim);
+
 	/* real output window */
 	update_window (pi, EMBEDDED, -1, -1);
 
@@ -1874,10 +1909,6 @@ vis_plugin_filter (GstPluginFeature *feature, gpointer data)
 {
 	GstElementFactory *f;
 
-	/* skip our fake visualizer */
-	if (strcmp (gst_plugin_feature_get_name (feature), "rbfakevis") == 0)
-		return FALSE;
-
 	if  (!GST_IS_ELEMENT_FACTORY (feature))
 		return FALSE;
 	f = GST_ELEMENT_FACTORY (feature);
@@ -2248,26 +2279,25 @@ static void
 create_controls (RBVisualizerPlugin *plugin)
 {
 	GtkWidget *widget;
-	GladeXML *xml;
-	char *gladefile;
+	GtkBuilder *builder;
+	char *builder_file;
 
-	/* load glade stuff */
-	gladefile = rb_plugin_find_file (RB_PLUGIN (plugin), "visualizer-controls.glade");
-	if (gladefile == NULL) {
+	/* load builder stuff */
+	builder_file = rb_plugin_find_file (RB_PLUGIN (plugin), "visualizer-controls.ui");
+	if (builder_file == NULL) {
 		return;
 	}
-	xml = glade_xml_new (gladefile,
-			     "visualizer_controls",
-			     NULL);
 
-	plugin->control_widget = glade_xml_get_widget (xml, "visualizer_controls");
-	plugin->element_combo = glade_xml_get_widget (xml, "element");
-	plugin->quality_combo = glade_xml_get_widget (xml, "quality");
-	plugin->mode_combo = glade_xml_get_widget (xml, "mode");
-	plugin->disable_button = glade_xml_get_widget (xml, "disable");
+	builder = rb_builder_load (builder_file, NULL);
 
-	plugin->screen_label = glade_xml_get_widget (xml, "screen_label");
-	plugin->screen_combo = glade_xml_get_widget (xml, "screen");
+	plugin->control_widget = GTK_WIDGET (gtk_builder_get_object (builder, "visualizer_controls"));
+	plugin->element_combo = GTK_WIDGET (gtk_builder_get_object (builder, "element"));
+	plugin->quality_combo = GTK_WIDGET (gtk_builder_get_object (builder, "quality"));
+	plugin->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode"));
+	plugin->disable_button = GTK_WIDGET (gtk_builder_get_object (builder, "disable"));
+
+	plugin->screen_label = GTK_WIDGET (gtk_builder_get_object (builder, "screen_label"));
+	plugin->screen_combo = GTK_WIDGET (gtk_builder_get_object (builder, "screen"));
 
 	populate_combo_boxes (plugin);
 
@@ -2288,32 +2318,28 @@ create_controls (RBVisualizerPlugin *plugin)
 				 0);
 
 	g_object_ref (plugin->control_widget);
-	g_object_unref (xml);
 
-	xml = glade_xml_new (gladefile,
-			     "play_controls",
-			     NULL);
-	plugin->play_control_widget = glade_xml_get_widget (xml, "play_controls");
-	plugin->song_info_label = glade_xml_get_widget (xml, "song_info");
+	plugin->play_control_widget = GTK_WIDGET (gtk_builder_get_object (builder, "play_controls"));
+	plugin->song_info_label = GTK_WIDGET (gtk_builder_get_object (builder, "song_info"));
 
-	plugin->play_button = glade_xml_get_widget (xml, "play");
+	plugin->play_button = GTK_WIDGET (gtk_builder_get_object (builder, "play"));
 	g_signal_connect_object (plugin->play_button, "toggled",
 				 G_CALLBACK (play_toggled_cb), plugin,
 				 0);
 
-	widget = glade_xml_get_widget (xml, "previous");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "previous"));
 	g_signal_connect_object (widget, "clicked",
 				 G_CALLBACK (previous_clicked_cb), plugin,
 				 0);
-	widget = glade_xml_get_widget (xml, "next");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "next"));
 	g_signal_connect_object (widget, "clicked",
 				 G_CALLBACK (next_clicked_cb), plugin,
 				 0);
 
 	g_object_ref (plugin->play_control_widget);
 
-	g_object_unref (xml);
-	g_free (gladefile);
+	g_object_unref (builder);
+	g_free (builder_file);
 }
 
 
