@@ -8,7 +8,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -36,7 +36,8 @@ typedef enum
 {
 	RB_PODCAST_PARSE_ERROR_FILE_INFO,		/* error getting podcast file info */
 	RB_PODCAST_PARSE_ERROR_MIME_TYPE,		/* podcast has unexpected mime type */
-	RB_PODCAST_PARSE_ERROR_XML_PARSE		/* error parsing podcast xml */
+	RB_PODCAST_PARSE_ERROR_XML_PARSE,		/* error parsing podcast xml */
+	RB_PODCAST_PARSE_ERROR_NO_ITEMS,		/* feed doesn't contain any downloadable items */
 } RBPodcastParseError;
 
 #define RB_PODCAST_PARSE_ERROR rb_podcast_parse_error_quark ()
