@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -27,12 +35,11 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <glib.h>
 
 #include "rb-podcast-properties-dialog.h"
 #include "rb-file-helpers.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-dialog.h"
 #include "rb-rating.h"
 #include "rb-util.h"
@@ -40,6 +47,7 @@
 
 static void rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass);
 static void rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog);
+static void rb_podcast_properties_dialog_dispose (GObject *object);
 static void rb_podcast_properties_dialog_finalize (GObject *object);
 static void rb_podcast_properties_dialog_set_property (GObject *object,
 						       guint prop_id,
@@ -118,6 +126,7 @@ rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass)
 					                      RB_TYPE_ENTRY_VIEW,
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	object_class->dispose = rb_podcast_properties_dialog_dispose;
 	object_class->finalize = rb_podcast_properties_dialog_finalize;
 
 	g_type_class_add_private (klass, sizeof (RBPodcastPropertiesDialogPrivate));
@@ -126,7 +135,8 @@ rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass)
 static void
 rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 {
-	GladeXML *xml;
+	GtkBuilder *builder;
+	AtkObject *lobj, *robj;
 
 	dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog,
 						    RB_TYPE_PODCAST_PROPERTIES_DIALOG,
@@ -144,50 +154,75 @@ rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 					 GTK_RESPONSE_OK);
 
-	xml = rb_glade_xml_new ("podcast-properties.glade",
-				"podcastproperties",
-				dialog);
-	glade_xml_signal_autoconnect (xml);
+	builder = rb_builder_load ("podcast-properties.ui", dialog);
 
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
-			   glade_xml_get_widget (xml, "podcastproperties"));
+			   GTK_WIDGET (gtk_builder_get_object (builder, "podcastproperties")));
 	dialog->priv->close_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
 							    GTK_STOCK_CLOSE,
 							    GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
-	/* get the widgets from the XML */
-	dialog->priv->title = glade_xml_get_widget (xml, "titleLabel");
-	dialog->priv->feed = glade_xml_get_widget (xml, "feedLabel");
-	dialog->priv->duration = glade_xml_get_widget (xml, "durationLabel");
-	dialog->priv->location = glade_xml_get_widget (xml, "locationLabel");
-	dialog->priv->download_location = glade_xml_get_widget (xml, "downloadLocationLabel");
-	dialog->priv->lastplayed = glade_xml_get_widget (xml, "lastplayedLabel");
-	dialog->priv->playcount = glade_xml_get_widget (xml, "playcountLabel");
-	dialog->priv->bitrate = glade_xml_get_widget (xml, "bitrateLabel");
-	dialog->priv->date = glade_xml_get_widget (xml, "dateLabel");
-	dialog->priv->description = glade_xml_get_widget (xml, "descriptionLabel");
+	/* get the widgets from the builder */
+	dialog->priv->title = GTK_WIDGET (gtk_builder_get_object (builder, "titleLabel"));
+	dialog->priv->feed = GTK_WIDGET (gtk_builder_get_object (builder, "feedLabel"));
+	dialog->priv->duration = GTK_WIDGET (gtk_builder_get_object (builder, "durationLabel"));
+	dialog->priv->location = GTK_WIDGET (gtk_builder_get_object (builder, "locationLabel"));
+	dialog->priv->download_location = GTK_WIDGET (gtk_builder_get_object (builder, "downloadLocationLabel"));
+	dialog->priv->lastplayed = GTK_WIDGET (gtk_builder_get_object (builder, "lastplayedLabel"));
+	dialog->priv->playcount = GTK_WIDGET (gtk_builder_get_object (builder, "playcountLabel"));
+	dialog->priv->bitrate = GTK_WIDGET (gtk_builder_get_object (builder, "bitrateLabel"));
+	dialog->priv->date = GTK_WIDGET (gtk_builder_get_object (builder, "dateLabel"));
+	dialog->priv->description = GTK_WIDGET (gtk_builder_get_object (builder, "descriptionLabel"));
 
-	rb_glade_boldify_label (xml, "titleDescLabel");
-	rb_glade_boldify_label (xml, "feedDescLabel");
-	rb_glade_boldify_label (xml, "locationDescLabel");
-	rb_glade_boldify_label (xml, "downloadLocationDescLabel");
-	rb_glade_boldify_label (xml, "durationDescLabel");
-	rb_glade_boldify_label (xml, "ratingDescLabel");
-	rb_glade_boldify_label (xml, "lastplayedDescLabel");
-	rb_glade_boldify_label (xml, "playcountDescLabel");
-	rb_glade_boldify_label (xml, "bitrateDescLabel");
-	rb_glade_boldify_label (xml, "dateDescLabel");
-	rb_glade_boldify_label (xml, "descriptionDescLabel");
+	rb_builder_boldify_label (builder, "titleDescLabel");
+	rb_builder_boldify_label (builder, "feedDescLabel");
+	rb_builder_boldify_label (builder, "locationDescLabel");
+	rb_builder_boldify_label (builder, "downloadLocationDescLabel");
+	rb_builder_boldify_label (builder, "durationDescLabel");
+	rb_builder_boldify_label (builder, "ratingDescLabel");
+	rb_builder_boldify_label (builder, "lastplayedDescLabel");
+	rb_builder_boldify_label (builder, "playcountDescLabel");
+	rb_builder_boldify_label (builder, "bitrateDescLabel");
+	rb_builder_boldify_label (builder, "dateDescLabel");
+	rb_builder_boldify_label (builder, "descriptionDescLabel");
 
 	dialog->priv->rating = GTK_WIDGET (rb_rating_new ());
 	g_signal_connect_object (dialog->priv->rating,
 				 "rated",
 				 G_CALLBACK (rb_podcast_properties_dialog_rated_cb),
 				 G_OBJECT (dialog), 0);
-	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "ratingVBox")),
+	gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (builder, "ratingVBox")),
 			   dialog->priv->rating);
-	g_object_unref (G_OBJECT (xml));
+
+	/* add relationship between the rating label and the rating widget */
+	lobj = gtk_widget_get_accessible (GTK_WIDGET (gtk_builder_get_object (builder, "ratingDescLabel")));
+	robj = gtk_widget_get_accessible (dialog->priv->rating);
+	
+	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
+	atk_object_add_relationship (robj, ATK_RELATION_LABELLED_BY, lobj);
+
+	g_object_unref (builder);
+}
+
+static void
+rb_podcast_properties_dialog_dispose (GObject *object)
+{
+	RBPodcastPropertiesDialog *dialog;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_PODCAST_PROPERTIES_DIALOG (object));
+
+	dialog = RB_PODCAST_PROPERTIES_DIALOG (object);
+
+	g_return_if_fail (dialog->priv != NULL);
+
+	if (dialog->priv->db != NULL) {
+		g_object_unref (dialog->priv->db);
+		dialog->priv->db = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_podcast_properties_dialog_parent_class)->dispose (object);
 }
 
 static void
@@ -202,10 +237,6 @@ rb_podcast_properties_dialog_finalize (GObject *object)
 
 	g_return_if_fail (dialog->priv != NULL);
 
-	if (dialog->priv->db != NULL) {
-		g_object_unref (dialog->priv->db);
-	}
-
 	G_OBJECT_CLASS (rb_podcast_properties_dialog_parent_class)->finalize (object);
 }
 
@@ -376,8 +407,10 @@ rb_podcast_properties_dialog_update_location (RBPodcastPropertiesDialog *dialog)
 	const char *location;
 	char *display;
 
-	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
-	display = gnome_vfs_format_uri_for_display (location);
+	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_MOUNTPOINT);
+	if (location == NULL)
+		location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
+	display = g_uri_unescape_string (location, NULL);
 	gtk_label_set_text (GTK_LABEL (dialog->priv->location), display);
 	g_free (display);
 }
@@ -390,7 +423,8 @@ rb_podcast_properties_dialog_update_download_location (RBPodcastPropertiesDialog
 	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_MOUNTPOINT);
 	if (location != NULL && location[0] != '\0') {
 		char *display;
-		display = gnome_vfs_format_uri_for_display (location);
+		location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
+		display = g_uri_unescape_string (location, NULL);
 		gtk_label_set_text (GTK_LABEL (dialog->priv->download_location), display);
 		g_free (display);
 	} else {
@@ -458,7 +492,7 @@ rb_podcast_properties_dialog_update_last_played (RBPodcastPropertiesDialog *dial
 	const char *str;
 
 	str = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LAST_PLAYED_STR);
-	gtk_label_set (GTK_LABEL (dialog->priv->lastplayed), str);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->lastplayed), str);
 }
 
 static void
@@ -479,8 +513,7 @@ rb_podcast_properties_dialog_update_date (RBPodcastPropertiesDialog *dialog)
 	post_time = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_POST_TIME);
 	time = rb_podcast_properties_dialog_parse_time (post_time);
 
-	gtk_label_set (GTK_LABEL (dialog->priv->date),
-		       time);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->date), time);
 	g_free (time);
 }
 
@@ -490,7 +523,7 @@ rb_podcast_properties_dialog_update_description (RBPodcastPropertiesDialog *dial
 	const char *str;
 
 	str = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_DESCRIPTION);
-	gtk_label_set (GTK_LABEL (dialog->priv->description), str);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->description), str);
 }
 
 static char *
