@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -41,10 +41,11 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
+#include <math.h>
 #include <glib/gprintf.h>
-#include <glib/gatomic.h>
+#include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkliststore.h>
+#include <gtk/gtk.h>
 #include <libxml/entities.h>
 #include <libxml/SAX.h>
 #include <libxml/parserInternals.h>
@@ -71,7 +72,7 @@ G_DEFINE_TYPE(RhythmDBTree, rhythmdb_tree, RHYTHMDB_TYPE)
 
 static void rhythmdb_tree_finalize (GObject *object);
 
-static gboolean rhythmdb_tree_load (RhythmDB *rdb, gboolean *die, GError **error);
+static gboolean rhythmdb_tree_load (RhythmDB *rdb, GCancellable *cancel, GError **error);
 static void rhythmdb_tree_save (RhythmDB *rdb);
 static void rhythmdb_tree_entry_new (RhythmDB *db, RhythmDBEntry *entry);
 static void rhythmdb_tree_entry_new_internal (RhythmDB *db, RhythmDBEntry *entry);
@@ -116,8 +117,8 @@ static void rhythmdb_hash_tree_foreach (RhythmDB *adb,
 					gpointer data);
 
 /* Update both of those! */
-#define RHYTHMDB_TREE_XML_VERSION "1.4"
-#define RHYTHMDB_TREE_XML_VERSION_INT 140
+#define RHYTHMDB_TREE_XML_VERSION "1.6"
+#define RHYTHMDB_TREE_XML_VERSION_INT 160
 
 static void destroy_tree_property (RhythmDBTreeProperty *prop);
 static RhythmDBTreeProperty *get_or_create_album (RhythmDBTree *db, RhythmDBTreeProperty *artist,
@@ -311,7 +312,7 @@ struct RhythmDBTreeLoadContext
 {
 	RhythmDBTree *db;
 	xmlParserCtxtPtr xmlctx;
-	gboolean *die;
+	GCancellable *cancel;
 	enum {
 		RHYTHMDB_TREE_PARSER_STATE_START,
 		RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB,
@@ -331,9 +332,10 @@ struct RhythmDBTreeLoadContext
 	GError **error;
 
 	/* updating */
-	gboolean has_date;
-	gboolean canonicalise_uris;
-	gboolean reload_all_metadata;
+	guint has_date : 1;
+	guint canonicalise_uris : 1;
+	guint reload_all_metadata : 1;
+	guint update_podcasts : 1;
 };
 
 /* Returns the version as an int, multiplied by 100,
@@ -347,7 +349,7 @@ version_to_int (const char *version)
 		return (int) (1.0 * 100);
 	}
 
-	return ver * 100;
+	return (int)roundf(ver * 100);
 }
 
 static void
@@ -355,7 +357,7 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 				    const char *name,
 				    const char **attrs)
 {
-	if (*ctx->die == TRUE) {
+	if (g_cancellable_is_cancelled (ctx->cancel)) {
 		xmlStopParser (ctx->xmlctx);
 		return;
 	}
@@ -375,6 +377,7 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 				if (!strcmp (*attrs, "version")) {
 					const char *version = *(attrs+1);
 
+					rb_debug ("loading database version %s (%d)", version, version_to_int (version));
 					switch (version_to_int (version)) {
 					case 100:
 					case 110:
@@ -384,11 +387,15 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 						rb_debug ("reloading all file metadata to get MusicBrainz tags (DB version 1.2)");
 						ctx->reload_all_metadata = TRUE;
 					case 130:
+					case 140:
 						/* Avoid being warned twice for very old DBs */
 						if (ctx->canonicalise_uris == FALSE) {
-							rb_debug ("old version of rhythmdb, performing URI canonicalisation for all entries (DB version 1.3)");
+							rb_debug ("old version of rhythmdb, performing URI canonicalisation for all entries (DB version %s)", version);
 							ctx->canonicalise_uris = TRUE;
 						}
+					case 150:
+						rb_debug ("Upgrade Podcasts remote vs. local locations");
+						ctx->update_podcasts = TRUE;
 					case RHYTHMDB_TREE_XML_VERSION_INT:
 						/* current version */
 						break;
@@ -397,8 +404,8 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 							g_set_error (ctx->error,
 								     RHYTHMDB_TREE_ERROR,
 								     RHYTHMDB_TREE_ERROR_DATABASE_TOO_NEW,
-								     _("The database was created by a later version of rhythmbox."
-								       "  This version of rhythmbox cannot read the database."));
+								     _("The database was created by a later version of Rhythmbox."
+								       "  This version of Rhythmbox cannot read the database."));
 							xmlStopParser (ctx->xmlctx);
 						}
 					}
@@ -482,7 +489,7 @@ static void
 rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx,
 				  const char *name)
 {
-	if (*ctx->die == TRUE) {
+	if (g_cancellable_is_cancelled (ctx->cancel)) {
 		xmlStopParser (ctx->xmlctx);
 		return;
 	}
@@ -499,7 +506,7 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx,
 		break;
 	case RHYTHMDB_TREE_PARSER_STATE_ENTRY:
 	{
-		if (!ctx->has_date | ctx->reload_all_metadata) {
+		if (!ctx->has_date || ctx->reload_all_metadata) {
 			/* there is no date metadata, so this is from an old version
 			 * reset the last-modified timestamp, so that the file is re-read
 			 */
@@ -517,6 +524,19 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx,
 				podcast->post_time = ctx->entry->last_seen;
 			}
 		}
+		if (ctx->entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
+			/* When upgrading Podcasts from 0.11.6 and prior, we need to
+			 * swap mountpoint and location if there is a mountpoint */
+			if (ctx->update_podcasts && ctx->entry->mountpoint != NULL) {
+				RBRefString *tmp;
+
+				rb_debug ("pre-Podcast avoidance found, swapping location/mountpoint");
+
+				tmp = ctx->entry->location;
+				ctx->entry->location = ctx->entry->mountpoint;
+				ctx->entry->mountpoint = tmp;
+			}
+		}
 
 		if (ctx->entry->location != NULL && rb_refstring_get (ctx->entry->location)[0] != '\0') {
 			RhythmDBEntry *entry;
@@ -530,9 +550,33 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx,
 					rhythmdb_commit (RHYTHMDB (ctx->db));
 					ctx->batch_count = 0;
 				}
+			} else if (ctx->entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST &&
+				   entry->type == RHYTHMDB_ENTRY_TYPE_SONG) {
+				rb_debug ("found song entry with duplicate location for Podcast post %s. merging metadata",
+					  rb_refstring_get (ctx->entry->location));
+
+				ctx->entry->play_count += entry->play_count;
+				if (ctx->entry->last_played < entry->last_played)
+					ctx->entry->last_played = entry->last_played;
+
+				/* Remove the song entry,
+				 * deleting requires relinquishing the locks */
+				g_mutex_unlock (ctx->db->priv->entries_lock);
+				rhythmdb_entry_delete (RHYTHMDB(ctx->db), entry);
+				g_mutex_lock (ctx->db->priv->entries_lock);
+				rhythmdb_commit (RHYTHMDB (ctx->db));
+
+				/* And add the Podcast entry to the database */
+				rhythmdb_tree_entry_new_internal (RHYTHMDB (ctx->db), ctx->entry);
+				rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
+				if (++ctx->batch_count == RHYTHMDB_QUERY_MODEL_SUGGESTED_UPDATE_CHUNK) {
+					rhythmdb_commit (RHYTHMDB (ctx->db));
+					ctx->batch_count = 0;
+				}
 			} else {
 				rb_debug ("found entry with duplicate location %s. merging metadata",
 					  rb_refstring_get (ctx->entry->location));
+
 				entry->play_count += ctx->entry->play_count;
 
 				if (entry->rating < 0.01)
@@ -597,11 +641,6 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx,
 				set = TRUE;
 			}
 			break;
-		case RHYTHMDB_PROP_MOUNTPOINT:
-			/* fix old podcast posts */
-			if (g_str_has_prefix (ctx->buf->str, "http://"))
-				skip = TRUE;
-			break;
 		default:
 			break;
 		}
@@ -653,7 +692,7 @@ rhythmdb_tree_parser_characters (struct RhythmDBTreeLoadContext *ctx,
 				 const char *data,
 				 guint len)
 {
-	if (*ctx->die == TRUE) {
+	if (g_cancellable_is_cancelled (ctx->cancel)) {
 		xmlStopParser (ctx->xmlctx);
 		return;
 	}
@@ -676,7 +715,7 @@ rhythmdb_tree_parser_characters (struct RhythmDBTreeLoadContext *ctx,
 
 static gboolean
 rhythmdb_tree_load (RhythmDB *rdb,
-		    gboolean *die,
+		    GCancellable *cancel,
 		    GError **error)
 {
 	RhythmDBTree *db = RHYTHMDB_TREE (rdb);
@@ -698,7 +737,7 @@ rhythmdb_tree_load (RhythmDB *rdb,
 
 	ctx->state = RHYTHMDB_TREE_PARSER_STATE_START;
 	ctx->db = db;
-	ctx->die = die;
+	ctx->cancel = cancel;
 	ctx->buf = g_string_sized_new (RHYTHMDB_TREE_PARSER_INITIAL_BUFFER_SIZE);
 	ctx->error = &local_error;
 
@@ -936,6 +975,9 @@ save_entry (RhythmDBTree *db,
 		case RHYTHMDB_PROP_ARTIST_SORTNAME:
 			save_entry_string(ctx, elt_name, rb_refstring_get (entry->artist_sortname));
 			break;
+		case RHYTHMDB_PROP_ALBUM_SORTNAME:
+			save_entry_string(ctx, elt_name, rb_refstring_get (entry->album_sortname));
+			break;
 		case RHYTHMDB_PROP_TRACK_NUMBER:
 			save_entry_ulong (ctx, elt_name, entry->tracknum, FALSE);
 			break;
@@ -1468,9 +1510,10 @@ rhythmdb_tree_entry_set (RhythmDB *adb,
 	type = entry->type;
 
 	/* don't process changes to entries we're loading, we'll get them
-	 * when the entry is complete.
+	 * when the entry is complete.  don't process changes for entries that
+	 * have been removed either.
 	 */
-	if (entry->flags & RHYTHMDB_ENTRY_TREE_LOADING)
+	if (entry->flags & (RHYTHMDB_ENTRY_TREE_LOADING | RHYTHMDB_ENTRY_TREE_REMOVED))
 		return FALSE;
 
 	/* Handle special properties */
@@ -1597,6 +1640,8 @@ rhythmdb_tree_entry_delete (RhythmDB *adb,
 	g_mutex_lock (db->priv->entries_lock);
 	g_assert (g_hash_table_remove (db->priv->entries, entry->location));
 	g_assert (g_hash_table_remove (db->priv->entry_ids, GINT_TO_POINTER (entry->id)));
+
+	entry->flags |= RHYTHMDB_ENTRY_TREE_REMOVED;
 	rhythmdb_entry_unref (entry);
 	g_mutex_unlock (db->priv->entries_lock);
 }
