@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -54,15 +54,13 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 
-#ifdef ENABLE_TRACK_TRANSFER
 #include <profiles/gnome-media-profiles.h>
 #include <profiles/audio-profile-choose.h>
-#endif
 
 #include "rhythmdb.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-file-helpers.h"
 #include "rb-util.h"
 #include "eel-gconf-extensions.h"
@@ -85,9 +83,7 @@ static char *impl_get_browser_key (RBSource *source);
 static char *impl_get_paned_key (RBBrowserSource *source);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_can_paste (RBSource *asource);
-#ifdef ENABLE_TRACK_TRANSFER
 static void impl_paste (RBSource *source, GList *entries);
-#endif
 static guint impl_want_uri (RBSource *source, const char *uri);
 static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
 
@@ -98,7 +94,6 @@ static void rb_library_source_library_location_changed (GConfClient *client,
 						    guint cnxn_id,
 						    GConfEntry *entry,
 						    RBLibrarySource *source);
-#ifdef ENABLE_TRACK_TRANSFER
 static void rb_library_source_layout_path_changed (GConfClient *client,
 						   guint cnxn_id,
 						   GConfEntry *entry,
@@ -109,7 +104,6 @@ static void rb_library_source_layout_filename_changed (GConfClient *client,
 						       RBLibrarySource *source);
 static void rb_library_source_edit_profile_clicked_cb (GtkButton *button,
 						       RBLibrarySource *source);
-#endif
 static void rb_library_source_ui_pref_changed (GConfClient *client,
 					       guint cnxn_id,
 					       GConfEntry *entry,
@@ -120,14 +114,13 @@ static gboolean rb_library_source_library_location_cb (GtkEntry *entry,
 static void rb_library_source_watch_toggled_cb (GtkToggleButton *button,
 						RBLibrarySource *source);
 static void rb_library_source_sync_child_sources (RBLibrarySource *source);
-#ifdef ENABLE_TRACK_TRANSFER
 static void rb_library_source_path_changed_cb (GtkComboBox *box,
 						RBLibrarySource *source);
 static void rb_library_source_filename_changed_cb (GtkComboBox *box,
 						   RBLibrarySource *source);
 static void rb_library_source_format_changed_cb (GtkWidget *widget,
 						 RBLibrarySource *source);
-#endif
+static void layout_example_label_update (RBLibrarySource *source);
 
 #define CONF_UI_LIBRARY_DIR CONF_PREFIX "/ui/library"
 #define CONF_STATE_LIBRARY_DIR CONF_PREFIX "/state/library"
@@ -135,7 +128,6 @@ static void rb_library_source_format_changed_cb (GtkWidget *widget,
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/library/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/library/show_browser"
 
-#ifdef ENABLE_TRACK_TRANSFER
 typedef struct {
 	char *title;
 	char *path;
@@ -159,7 +151,6 @@ const LibraryPathElement library_layout_filenames[] = {
 	{N_("Number. Artist - Title"), "%tN. %ta - %tt"},
 };
 const int num_library_layout_filenames = G_N_ELEMENTS (library_layout_filenames);
-#endif
 
 struct RBLibrarySourcePrivate
 {
@@ -174,19 +165,15 @@ struct RBLibrarySourcePrivate
 
 	GtkWidget *library_location_entry;
 	GtkWidget *watch_library_check;
-#ifdef ENABLE_TRACK_TRANSFER
 	GtkWidget *layout_path_menu;
 	GtkWidget *layout_filename_menu;
 	GtkWidget *preferred_format_menu;
 	GtkWidget *layout_example_label;
-#endif
 
 	guint library_location_notify_id;
 	guint ui_dir_notify_id;
-#ifdef ENABLE_TRACK_TRANSFER
 	guint layout_path_notify_id;
 	guint layout_filename_notify_id;
-#endif
 };
 
 #define RB_LIBRARY_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LIBRARY_SOURCE, RBLibrarySourcePrivate))
@@ -209,9 +196,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) impl_can_paste;
-#ifdef ENABLE_TRACK_TRANSFER
 	source_class->impl_paste = impl_paste;
-#endif
 	source_class->impl_want_uri = impl_want_uri;
 	source_class->impl_add_uri = impl_add_uri;
 
@@ -220,9 +205,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	g_type_class_add_private (klass, sizeof (RBLibrarySourcePrivate));
 
-#ifdef ENABLE_TRACK_TRANSFER
 	gnome_media_profiles_init (eel_gconf_client_get_global ());
-#endif
 }
 
 static void
@@ -256,7 +239,7 @@ rb_library_source_dispose (GObject *object)
 		eel_gconf_notification_remove (source->priv->library_location_notify_id);
 		source->priv->library_location_notify_id = 0;
 	}
-#ifdef ENABLE_TRACK_TRANSFER
+
 	if (source->priv->layout_path_notify_id != 0) {
 		eel_gconf_notification_remove (source->priv->layout_path_notify_id);
 		source->priv->layout_path_notify_id = 0;
@@ -266,7 +249,6 @@ rb_library_source_dispose (GObject *object)
 		eel_gconf_notification_remove (source->priv->layout_filename_notify_id);
 		source->priv->layout_filename_notify_id = 0;
 	}
-#endif
 
 	G_OBJECT_CLASS (rb_library_source_parent_class)->dispose (object);
 }
@@ -298,6 +280,13 @@ add_child_sources_idle (RBLibrarySource *source)
 	return FALSE;
 }
 
+static void
+db_load_complete_cb (RhythmDB *db, RBLibrarySource *source)
+{
+	/* once the database is loaded, we can run the query to populate the library source */
+	g_object_set (source, "populate", TRUE, NULL);
+}
+
 static GObject *
 rb_library_source_constructor (GType type,
 			       guint n_construct_properties,
@@ -314,6 +303,8 @@ rb_library_source_constructor (GType type,
 	g_object_get (source, "shell", &shell, NULL);
 	g_object_get (shell, "db", &source->priv->db, NULL);
 
+	g_signal_connect_object (source->priv->db, "load-complete", G_CALLBACK (db_load_complete_cb), source, 0);
+
 	rb_library_source_ui_prefs_sync (source);
 
 	/* Set up the default library location if there's no library location set */
@@ -326,6 +317,28 @@ rb_library_source_constructor (GType type,
 			list = g_slist_prepend (list, music_dir_uri);
 			eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
 		}
+	} else {
+		/* ensure all library locations are URIs and not file paths */
+		GSList *t;
+		gboolean update = FALSE;
+		for (t = list; t != NULL; t = t->next) {
+			char *location;
+
+			location = (char *)t->data;
+			if (location[0] == '/') {
+				char *uri = g_filename_to_uri (location, NULL, NULL);
+				if (uri != NULL) {
+					rb_debug ("converting library location path %s to URI %s", location, uri);
+					g_free (location);
+					t->data = uri;
+					update = TRUE;
+				}
+			}
+		}
+
+		if (update) {
+			eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+		}
 	}
 	rb_slist_deep_free (list);
 
@@ -380,6 +393,7 @@ rb_library_source_new (RBShell *shell)
 					  "sorting-key", CONF_STATE_LIBRARY_SORTING,
 					  "shell", shell,
 					  "icon", icon,
+					  "populate", FALSE,		/* wait until the database is loaded */
 					  NULL));
 	if (icon != NULL) {
 		g_object_unref (icon);
@@ -390,7 +404,6 @@ rb_library_source_new (RBShell *shell)
 	return source;
 }
 
-#ifdef ENABLE_TRACK_TRANSFER
 static void
 rb_library_source_edit_profile_clicked_cb (GtkButton *button, RBLibrarySource *source)
 {
@@ -402,7 +415,6 @@ rb_library_source_edit_profile_clicked_cb (GtkButton *button, RBLibrarySource *s
 	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 }
-#endif
 
 static void
 rb_library_source_location_button_clicked_cb (GtkButton *button, RBLibrarySource *source)
@@ -420,7 +432,7 @@ rb_library_source_location_button_clicked_cb (GtkButton *button, RBLibrarySource
 			uri = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dialog));
 		}
 
-		path = gnome_vfs_format_uri_for_display (uri);
+		path = g_uri_unescape_string (uri, NULL);
 
 		gtk_entry_set_text (GTK_ENTRY (source->priv->library_location_entry), path);
 		rb_library_source_library_location_cb (GTK_ENTRY (source->priv->library_location_entry),
@@ -436,12 +448,10 @@ static GtkWidget *
 impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	GtkWidget *tmp;
-	GladeXML *xml;
-#ifdef ENABLE_TRACK_TRANSFER
-	GtkWidget *label;
+	GtkBuilder *builder;
+	GObject *tmp;
+	GObject *label;
 	int i;
-#endif
 
 	if (source->priv->config_widget)
 		return source->priv->config_widget;
@@ -449,15 +459,15 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	g_object_ref (G_OBJECT (prefs));
 	source->priv->shell_prefs = prefs;
 
-	xml = rb_glade_xml_new ("library-prefs.glade", "library_vbox", source);
+	builder = rb_builder_load ("library-prefs.ui", source);
 	source->priv->config_widget =
-		glade_xml_get_widget (xml, "library_vbox");
+		GTK_WIDGET (gtk_builder_get_object (builder, "library_vbox"));
 
-	rb_glade_boldify_label (xml, "library_location_label");
+	rb_builder_boldify_label (builder, "library_location_label");
 
-	source->priv->library_location_entry = glade_xml_get_widget (xml, "library_location_entry");
-	tmp = glade_xml_get_widget (xml, "library_location_button");
-	g_signal_connect (G_OBJECT (tmp),
+	source->priv->library_location_entry = GTK_WIDGET (gtk_builder_get_object (builder, "library_location_entry"));
+	tmp = gtk_builder_get_object (builder, "library_location_button");
+	g_signal_connect (tmp,
 			  "clicked",
 			  G_CALLBACK (rb_library_source_location_button_clicked_cb),
 			  asource);
@@ -466,19 +476,18 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 			  G_CALLBACK (rb_library_source_library_location_cb),
 			  asource);
 
-	source->priv->watch_library_check = glade_xml_get_widget (xml, "watch_library_check");
+	source->priv->watch_library_check = GTK_WIDGET (gtk_builder_get_object (builder, "watch_library_check"));
 	g_signal_connect (G_OBJECT (source->priv->watch_library_check),
 			  "toggled",
 			  G_CALLBACK (rb_library_source_watch_toggled_cb),
 			  asource);
 
-#ifdef ENABLE_TRACK_TRANSFER
-	rb_glade_boldify_label (xml, "library_structure_label");
+	rb_builder_boldify_label (builder, "library_structure_label");
 
-	tmp = glade_xml_get_widget (xml, "layout_path_menu_box");
-	label = glade_xml_get_widget (xml, "layout_path_menu_label");
+	tmp = gtk_builder_get_object (builder, "layout_path_menu_box");
+	label = gtk_builder_get_object (builder, "layout_path_menu_label");
 	source->priv->layout_path_menu = gtk_combo_box_new_text ();
-	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->layout_path_menu);
+	gtk_box_pack_start (GTK_BOX (tmp), source->priv->layout_path_menu, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->layout_path_menu);
 	g_signal_connect (G_OBJECT (source->priv->layout_path_menu),
 			  "changed",
@@ -489,10 +498,10 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 					   _(library_layout_paths[i].title));
 	}
 
-	tmp = glade_xml_get_widget (xml, "layout_filename_menu_box");
-	label = glade_xml_get_widget (xml, "layout_filename_menu_label");
+	tmp = gtk_builder_get_object (builder, "layout_filename_menu_box");
+	label = gtk_builder_get_object (builder, "layout_filename_menu_label");
 	source->priv->layout_filename_menu = gtk_combo_box_new_text ();
-	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->layout_filename_menu);
+	gtk_box_pack_start (GTK_BOX (tmp), source->priv->layout_filename_menu, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->layout_filename_menu);
 	g_signal_connect (G_OBJECT (source->priv->layout_filename_menu),
 			  "changed",
@@ -503,30 +512,23 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 					   _(library_layout_filenames[i].title));
 	}
 
-	tmp = glade_xml_get_widget (xml, "edit_profile_button");
-	g_signal_connect (G_OBJECT (tmp),
+	tmp = gtk_builder_get_object (builder, "edit_profile_button");
+	g_signal_connect (tmp,
 			  "clicked",
 			  G_CALLBACK (rb_library_source_edit_profile_clicked_cb),
 			  asource);
 
-	tmp = glade_xml_get_widget (xml, "preferred_format_menu_box");
-	label = glade_xml_get_widget (xml, "preferred_format_menu_label");
+	tmp = gtk_builder_get_object (builder, "preferred_format_menu_box");
+	label = gtk_builder_get_object (builder, "preferred_format_menu_label");
 	source->priv->preferred_format_menu = gm_audio_profile_choose_new ();
-	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->preferred_format_menu);
+	gtk_box_pack_start (GTK_BOX (tmp), source->priv->preferred_format_menu, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->preferred_format_menu);
 	g_signal_connect (G_OBJECT (source->priv->preferred_format_menu),
 			  "changed",
 			  G_CALLBACK (rb_library_source_format_changed_cb),
 			  asource);
 
-	source->priv->layout_example_label = glade_xml_get_widget (xml, "layout_example_label");
-#else
-	tmp = glade_xml_get_widget (xml, "library_structure_vbox");
-	gtk_widget_set_no_show_all (tmp, TRUE);
-	gtk_widget_hide (tmp);
-#endif
-
-	g_object_unref (G_OBJECT (xml));
+	source->priv->layout_example_label = GTK_WIDGET (gtk_builder_get_object (builder, "layout_example_label"));
 
 	rb_library_source_preferences_sync (source);
 
@@ -566,10 +568,8 @@ static void
 rb_library_source_preferences_sync (RBLibrarySource *source)
 {
 	GSList *list;
-#ifdef ENABLE_TRACK_TRANSFER
 	char *str;
 	GConfClient *gconf_client;
-#endif
 
 	rb_debug ("syncing pref dialog state");
 
@@ -586,7 +586,7 @@ rb_library_source_preferences_sync (RBLibrarySource *source)
 
 		gtk_widget_set_sensitive (source->priv->library_location_entry, TRUE);
 
-		path = gnome_vfs_format_uri_for_display (list->data);
+		path = g_uri_unescape_string (list->data, NULL);
 		gtk_entry_set_text (GTK_ENTRY (source->priv->library_location_entry), path);
 		g_free (path);
 	} else if (g_slist_length (list) == 0) {
@@ -609,7 +609,6 @@ rb_library_source_preferences_sync (RBLibrarySource *source)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (source->priv->watch_library_check),
 				      eel_gconf_get_boolean (CONF_MONITOR_LIBRARY));
 
-#ifdef ENABLE_TRACK_TRANSFER
 	/* preferred format */
 	str = eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT);
 	if (str) {
@@ -633,7 +632,6 @@ rb_library_source_preferences_sync (RBLibrarySource *source)
 	rb_library_source_layout_filename_changed (gconf_client, -1,
 						   gconf_client_get_entry (gconf_client, CONF_LIBRARY_LAYOUT_FILENAME, NULL, TRUE, NULL),
 						   source);
-#endif
 }
 
 static gboolean
@@ -643,10 +641,13 @@ rb_library_source_library_location_cb (GtkEntry *entry,
 {
 	GSList *list = NULL;
 	const char *path;
+	GFile *file;
 	char *uri;
 
 	path = gtk_entry_get_text (entry);
-	uri = gnome_vfs_make_uri_from_input (path);
+	file = g_file_parse_name (path);
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
 
 	if (uri && uri[0])
 		list = g_slist_prepend (NULL, (gpointer)uri);
@@ -728,7 +729,6 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-#ifdef ENABLE_TRACK_TRANSFER
 static void
 rb_library_source_path_changed_cb (GtkComboBox *box, RBLibrarySource *source)
 {
@@ -758,19 +758,21 @@ rb_library_source_format_changed_cb (GtkWidget *widget, RBLibrarySource *source)
 
 	profile = gm_audio_profile_choose_get_active (widget);
 	eel_gconf_set_string (CONF_LIBRARY_PREFERRED_FORMAT, gm_audio_profile_get_id (profile));
+	
+	layout_example_label_update (source);
 }
 
 /**
  * Perform magic on a path to make it safe.
  *
- * This will always replace '/' with ' ', and optionally make the file name
- * shell-friendly. This involves removing [?*\ ] and replacing with '_'.  Also
- * any leading periods are removed so that the files don't end up being hidden.
+ * This will always replace '/' with '-', and optionally make the file name
+ * shell-friendly. This involves removing replacing shell metacharacters and all
+ * whitespace with '_'. Also any leading periods are removed so that the files
+ * don't end up being hidden.
  */
 static char *
 sanitize_path (const char *str)
 {
-	gchar *res = NULL;
 	gchar *s;
 
 	/* Skip leading periods, otherwise files disappear... */
@@ -789,9 +791,8 @@ sanitize_path (const char *str)
 		/* TODO: I'd like this to compress whitespace aswell */
 		g_strdelimit (s, "\t ", '_');
 	}
-	res = g_filename_from_utf8(s, -1, NULL, NULL, NULL);
-	g_free(s);
-	return res ? res : g_strdup(str);
+
+	return s;
 }
 
 static char *
@@ -809,8 +810,7 @@ sanitize_pattern (const char *pat)
 }
 
 /*
- * Parse a filename pattern and replace markers with values from a TrackDetails
- * structure.
+ * Parse a filename pattern and replace markers with values from a RhythmDBEntry
  *
  * Valid markers so far are:
  * %at -- album title
@@ -827,22 +827,36 @@ sanitize_pattern (const char *pat)
  * %tS -- track artist sortname (lowercase)
  */
 static char *
-filepath_parse_pattern (const char *pattern,
+filepath_parse_pattern (RhythmDB *db,
+			const char *pattern,
 			RhythmDBEntry *entry)
 {
 	/* p is the pattern iterator, i is a general purpose iterator */
 	const char *p;
 	char *temp;
 	GString *s;
+	GValue *value;
+	RBRefString *albumartist;
 
 	if (pattern == NULL || pattern[0] == 0)
 		return g_strdup (" ");
 
+	/* request album artist (this is sort of temporary) */
+	value = rhythmdb_entry_request_extra_metadata (db, entry, RHYTHMDB_PROP_ALBUM_ARTIST);
+	if (value != NULL) {
+		albumartist = rb_refstring_new (g_value_get_string (value));
+		g_value_unset (value);
+		g_free (value);
+	} else {
+		albumartist = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ARTIST);
+	}
+
 	s = g_string_new (NULL);
 
 	p = pattern;
 	while (*p) {
 		char *string = NULL;
+		char *t;
 
 		/* If not a % marker, copy and continue */
 		if (*p != '%') {
@@ -868,22 +882,22 @@ filepath_parse_pattern (const char *pattern,
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM));
 				break;
 			case 'T':
-				temp = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_FOLDED));
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_FOLDED));
 				break;
 			case 'a':
-				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST));
+				string = sanitize_path (rb_refstring_get (albumartist));
 				break;
 			case 'A':
-				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_FOLDED));
+				string = sanitize_path (rb_refstring_get_folded (albumartist));
 				break;
-			/*case 's':
-				string = sanitize_path (album sort name);
+			case 's':
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
 				break;
 			case 'S':
-				char *t = g_utf8_strdown (album sort name)
+				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
 				string = sanitize_path (t);
 				g_free (t);
-				break;*/
+				break;
 			default:
 				string = g_strdup_printf ("%%a%c", *p);
 			}
@@ -907,14 +921,14 @@ filepath_parse_pattern (const char *pattern,
 			case 'A':
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_FOLDED));
 				break;
-			/*case 's':
-				string = sanitize_path (artist sort name);
+			case 's':
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
 				break;
 			case 'S':
-				char *t = g_utf8_strdown (artist sort name)
+				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
 				string = sanitize_path (t);
 				g_free (t);
-				break;*/
+				break;
 			case 'n':
 				/* Track number */
 				string = g_strdup_printf ("%u", (guint)rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER));
@@ -942,6 +956,7 @@ filepath_parse_pattern (const char *pattern,
 
 	temp = s->str;
 	g_string_free (s, FALSE);
+	rb_refstring_unref (albumartist);
 	return temp;
 }
 
@@ -979,8 +994,8 @@ layout_example_label_update (RBLibrarySource *source)
 	sample_entry = rhythmdb_entry_example_new (source->priv->db, entry_type, NULL);
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
-	file_value = filepath_parse_pattern (file_pattern, sample_entry);
-	path_value = filepath_parse_pattern (path_pattern, sample_entry);
+	file_value = filepath_parse_pattern (source->priv->db, file_pattern, sample_entry);
+	path_value = filepath_parse_pattern (source->priv->db, path_pattern, sample_entry);
 	rhythmdb_entry_unref (sample_entry);
 
 	example = g_build_filename (G_DIR_SEPARATOR_S, path_value, file_value, NULL);
@@ -1076,8 +1091,9 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 static char*
 build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 {
-	GnomeVFSURI *uri;
-	GnomeVFSURI *new;
+	GFile *library_location;
+	GFile *dir;
+	GFile *dest;
 	char *realfile;
 	char *realpath;
 	char *filename;
@@ -1104,15 +1120,11 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	g_free (layout_filename);
 	layout_filename = tmp;
 
-	uri = gnome_vfs_uri_new ((const char *)list->data);
-	if (uri == NULL) {
-		goto out;
-	}
+	realpath = filepath_parse_pattern (source->priv->db, layout_path, entry);
 
-	realpath = filepath_parse_pattern (layout_path, entry);
-	new = gnome_vfs_uri_append_path (uri, realpath);
-	gnome_vfs_uri_unref (uri);
-	uri = new;
+	library_location = g_file_new_for_uri ((const char *)list->data);
+	dir = g_file_resolve_relative_path (library_location, realpath);
+	g_object_unref (library_location);
 	g_free (realpath);
 
 	if (g_str_has_prefix (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE), "audio/x-raw")) {
@@ -1142,7 +1154,7 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 			*tmp = '\0';
 	}
 
-	realfile = filepath_parse_pattern (layout_filename, entry);
+	realfile = filepath_parse_pattern (source->priv->db, layout_filename, entry);
 	if (extension) {
 		filename = g_strdup_printf ("%s.%s", realfile, extension);
 		g_free (realfile);
@@ -1150,14 +1162,13 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 		filename = realfile;
 	}
 
-	new = gnome_vfs_uri_append_file_name (uri, filename);
-	gnome_vfs_uri_unref (uri);
-	uri = new;
+	dest = g_file_resolve_relative_path (dir, filename);
+	g_object_unref (dir);
 	g_free (extension);
 	g_free (filename);
 
-	string = gnome_vfs_uri_to_string (uri, 0);
-	gnome_vfs_uri_unref (uri);
+	string = g_file_get_uri (dest);
+	g_object_unref (dest);
  out:
 	rb_slist_deep_free (list);
 	g_free (layout_path);
@@ -1166,12 +1177,10 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 
 	return string;
 }
-#endif
 
 static gboolean
 impl_can_paste (RBSource *asource)
 {
-#ifdef ENABLE_TRACK_TRANSFER
 	GSList *list;
 	gboolean can_paste = TRUE;
 	char *str;
@@ -1192,12 +1201,8 @@ impl_can_paste (RBSource *asource)
 	can_paste &= (str != NULL);
 	g_free (str);
 	return can_paste;
-#else
-	return FALSE;
-#endif
 }
 
-#ifdef ENABLE_TRACK_TRANSFER
 static void
 completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, RBLibrarySource *source)
 {
@@ -1233,18 +1238,22 @@ impl_paste (RBSource *asource, GList *entries)
 		RhythmDBEntryType entry_type;
 		RBSource *source_source;
 		char *dest;
+		char *sane_dest;
 
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
 		entry_type = rhythmdb_entry_get_entry_type (entry);
-		if (entry_type == source_entry_type)
-			/* copying to ourselves would be silly */
+		if (entry_type == source_entry_type) {
+			rb_debug ("can't copy an entry from the library to itself");
 			continue;
+		}
 
 		/* see if the responsible source lets us copy */
 		source_source = rb_shell_get_source_by_entry_type (shell, entry_type);
-		if ((source_source != NULL) && !rb_source_can_copy (source_source))
+		if ((source_source != NULL) && !rb_source_can_copy (source_source)) {
+			rb_debug ("source for the entry doesn't want us to copy it");
 			continue;
+		}
 
 		dest = build_filename (source, entry);
 		if (dest == NULL) {
@@ -1252,15 +1261,17 @@ impl_paste (RBSource *asource, GList *entries)
 			continue;
 		}
 
+		sane_dest = rb_sanitize_uri_for_filesystem (dest);
+		g_free (dest);
+
 		rb_removable_media_manager_queue_transfer (rm_mgr, entry,
-							  dest, NULL,
-							  (RBTranferCompleteCallback)completed_cb, source);
+							  sane_dest, NULL,
+							  (RBTransferCompleteCallback)completed_cb, source);
 	}
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
 
 	g_object_unref (rm_mgr);
 }
-#endif
 
 static guint
 impl_want_uri (RBSource *source, const char *uri)
@@ -1291,26 +1302,21 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	RBSource *source;
 	GPtrArray *query;
 	RBShell *shell;
-	GnomeVFSURI *uri;
 	char *name;
 	GdkPixbuf *icon;
 	RhythmDBEntryType entry_type;
 	char *sort_column;
 	int sort_order;
+	GFile *file;
 
 	g_object_get (library_source,
 		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
-	uri = gnome_vfs_uri_new (path);
-	if (uri == NULL) {
-		g_object_unref (shell);
-		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
-		return;
-	}
 
-	name = gnome_vfs_uri_extract_short_name (uri);
-	gnome_vfs_uri_unref (uri);
+	file = g_file_new_for_uri (path);		/* ? */
+	name = g_file_get_basename (file);
+	g_object_unref (file);
 
 	rb_entry_view_get_sorting_order (rb_source_get_entry_view (RB_SOURCE (library_source)),
 					 &sort_column, &sort_order);
