@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -37,7 +37,6 @@
 #include <libxml/tree.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
 #include <totem-pl-parser.h>
 
 #include "rb-entry-view.h"
@@ -55,6 +54,8 @@
 #include "rb-static-playlist-source.h"
 #include "rb-auto-playlist-source.h"
 
+#include "rb-playlist-manager.h"
+
 /**
  * SECTION:rb-playlist-source
  * @short_description: Base class for playlist sources
@@ -174,7 +175,6 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_get_browser_key = impl_get_browser_key;
 	source_class->impl_get_entry_view = impl_get_entry_view;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
@@ -356,6 +356,7 @@ rb_playlist_source_constructor (GType type,
  	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_QUALITY, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LOCATION, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN, FALSE);
 	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
@@ -585,19 +586,20 @@ playlist_iter_func (GtkTreeModel *model,
  * rb_playlist_source_save_playlist:
  * @source: a #RBPlaylistSource
  * @uri: destination URI
- * @m3u_format: if TRUE, save as .m3u, otherwise .pls
+ * @export_type: format to save in
  *
  * Saves the playlist to an external file in a standard
- * format (M3U or PLS).
+ * format (M3U, PLS, or XSPF).
  */
 void
 rb_playlist_source_save_playlist (RBPlaylistSource *source,
 				  const char *uri,
-				  gboolean m3u_format)
+				  RBPlaylistExportType export_type)
 {
 	TotemPlParser *playlist;
 	GError *error = NULL;
 	char *name;
+	gint totem_format;
 
 	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
 
@@ -606,9 +608,22 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 
 	g_object_get (source, "name", &name, NULL);
 
+	switch (export_type) {
+	case RB_PLAYLIST_EXPORT_TYPE_XSPF:
+		totem_format = TOTEM_PL_PARSER_XSPF;
+		break;
+	case RB_PLAYLIST_EXPORT_TYPE_M3U:
+		totem_format = TOTEM_PL_PARSER_M3U;
+		break;
+	case RB_PLAYLIST_EXPORT_TYPE_PLS:
+	default:
+		totem_format = TOTEM_PL_PARSER_PLS;
+		break;
+	}
+
 	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
 					  playlist_iter_func, uri, name,
-					  m3u_format ? TOTEM_PL_PARSER_M3U : TOTEM_PL_PARSER_PLS,
+					  totem_format,
 					  NULL, &error);
 	g_object_unref (playlist);
 	g_free (name);
